@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
- * Copyright (c) 2002-2007 Mellanox Technologies LTD. All rights reserved.
+ * Copyright (c) 2002-2011 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
  *
  * This software is available to you under a choice of one of two
@@ -66,6 +66,7 @@ BEGIN_C_DECLS
 struct osm_port;
 struct osm_node;
 struct osm_mgrp;
+struct osm_sm;
 
 /****h* OpenSM/Physical Port
 * NAME
@@ -104,7 +105,9 @@ struct osm_mgrp;
 */
 typedef struct osm_physp {
 	ib_port_info_t port_info;
+	ib_mlnx_ext_port_info_t ext_port_info;
 	ib_net64_t port_guid;
+	ib_net64_t (*p_guids)[];
 	uint8_t port_num;
 	struct osm_node *p_node;
 	struct osm_physp *p_remote_physp;
@@ -117,16 +120,33 @@ typedef struct osm_physp {
 	ib_vl_arb_table_t vl_arb[4];
 	cl_ptr_vector_t slvl_by_port;
 	uint8_t hop_wf;
+	union {
+		struct {
+			ib_sw_cong_setting_t sw_cong_setting;
+		} sw;
+		struct {
+			ib_ca_cong_setting_t ca_cong_setting;
+			ib_cc_tbl_t cc_tbl[OSM_CCT_ENTRY_MAD_BLOCKS];
+		} ca;
+	} cc;
 } osm_physp_t;
 /*
 * FIELDS
 *	port_info
 *		The IBA defined PortInfo data for this port.
 *
+*	ext_port_info
+*		Mellanox vendor specific ExtendedPortInfo data for this port.
+*
 *	port_guid
 *		Port GUID value of this port.  For switches,
 *		all ports share the same GUID value.
 *
+*	p_guids
+*		Pointer to array of GUIDs obtained from GUIDInfo.
+*		This pointer is null for switch physical/external ports
+*		(used for endports only).
+*
 *	port_num
 *		The port number of this port.  The PortInfo also
 *		contains a port_number, but that number is not
@@ -176,6 +196,15 @@ typedef struct osm_physp {
 *	hop_wf
 *		Hop weighting factor to be used in the routing.
 *
+*	sw_cong_setting
+*		Physical port switch congestion settings (switches only)
+*
+*	ca_cong_setting
+*		Physical port ca congestion settings (cas only)
+*
+*	cc_tbl
+*		Physical port ca congestion control table (cas only)
+*
 * SEE ALSO
 *	Port
 *********/
@@ -235,7 +264,7 @@ void osm_physp_init(IN osm_physp_t * p_physp, IN ib_net64_t port_guid,
 *
 *	h_bind
 *		[in] Bind handle on which this port is accessed.
-*		Caller should use OSM_INVALID_BIND_HANDLE if the bind
+*		Caller should use OSM_BIND_INVALID_HANDLE if the bind
 *		handle to this port is unknown.
 *
 *	hop_count
@@ -421,22 +450,9 @@ static inline void osm_physp_set_health(IN osm_physp_t * p_physp,
 *
 * SYNOPSIS
 */
-static inline void osm_physp_set_port_info(IN osm_physp_t * p_physp,
-					   IN const ib_port_info_t * p_pi)
-{
-	CL_ASSERT(p_pi);
-	CL_ASSERT(osm_physp_is_valid(p_physp));
-
-	if (ib_port_info_get_port_state(p_pi) == IB_LINK_DOWN) {
-		/* If PortState is down, only copy PortState */
-		/* and PortPhysicalState per C14-24-2.1 */
-		ib_port_info_set_port_state(&p_physp->port_info, IB_LINK_DOWN);
-		ib_port_info_set_port_phys_state
-		    (ib_port_info_get_port_phys_state(p_pi),
-		     &p_physp->port_info);
-	} else
-		p_physp->port_info = *p_pi;
-}
+void osm_physp_set_port_info(IN osm_physp_t * p_physp,
+					   IN const ib_port_info_t * p_pi,
+					   IN const struct osm_sm * p_sm);
 
 /*
 * PARAMETERS
@@ -446,6 +462,9 @@ static inline void osm_physp_set_port_info(IN osm_physp_t * p_physp,
 *	p_pi
 *		[in] Pointer to the IBA defined PortInfo at this port number.
 *
+*	p_sm
+*		[in] Pointer to an osm_sm_t object.
+*
 * RETURN VALUES
 *	This function does not return a value.
 *
@@ -549,7 +568,7 @@ static inline void osm_physp_set_slvl_tbl(IN osm_physp_t * p_physp,
 
 	CL_ASSERT(p_slvl_tbl);
 	CL_ASSERT(osm_physp_is_valid(p_physp));
-	p_tbl = cl_ptr_vector_get(&p_physp->slvl_by_port, in_port_num);
+	p_tbl = (ib_slvl_table_t *)cl_ptr_vector_get(&p_physp->slvl_by_port, in_port_num);
 	*p_tbl = *p_slvl_tbl;
 }
 
@@ -590,7 +609,7 @@ static inline ib_slvl_table_t *osm_physp_get_slvl_tbl(IN const osm_physp_t *
 	ib_slvl_table_t *p_tbl;
 
 	CL_ASSERT(osm_physp_is_valid(p_physp));
-	p_tbl = cl_ptr_vector_get(&p_physp->slvl_by_port, in_port_num);
+	p_tbl = (ib_slvl_table_t *)cl_ptr_vector_get(&p_physp->slvl_by_port, in_port_num);
 	return p_tbl;
 }
 
@@ -1220,7 +1239,7 @@ osm_port_t *osm_port_new(IN const ib_node_info_t * p_ni,
 /*
 * PARAMETERS
 *	p_ni
-*		[in] Pointer to the NodeInfo attribute relavent for this port.
+*		[in] Pointer to the NodeInfo attribute relevant for this port.
 *
 *	p_parent_node
 *		[in] Pointer to the initialized parent osm_node_t object
@@ -1367,7 +1386,8 @@ void osm_port_get_lid_range_ho(IN const osm_port_t * p_port,
 * SYNOPSIS
 */
 uint8_t osm_physp_calc_link_mtu(IN osm_log_t * p_log,
-				IN const osm_physp_t * p_physp);
+				IN const osm_physp_t * p_physp,
+				IN uint8_t current_mtu);
 /*
 * PARAMETERS
 *	p_log
@@ -1376,6 +1396,9 @@ uint8_t osm_physp_calc_link_mtu(IN osm_log_t * p_log,
 *	p_physp
 *		[in] Pointer to an osm_physp_t object.
 *
+*	current_mtu
+*		[in] Current neighbor mtu on this port
+*
 * RETURN VALUES
 *	The MTU of the link to be used.
 *
@@ -1397,7 +1420,8 @@ uint8_t osm_physp_calc_link_mtu(IN osm_log_t * p_log,
 */
 uint8_t osm_physp_calc_link_op_vls(IN osm_log_t * p_log,
 				   IN const osm_subn_t * p_subn,
-				   IN const osm_physp_t * p_physp);
+				   IN const osm_physp_t * p_physp,
+				   IN uint8_t current_op_vls);
 /*
 * PARAMETERS
 *	p_log
@@ -1409,6 +1433,9 @@ uint8_t osm_physp_calc_link_op_vls(IN osm_log_t * p_log,
 *	p_physp
 *		[in] Pointer to an osm_physp_t object.
 *
+*	current_op_vls
+*		[in] Current operational VL on the port
+*
 * RETURN VALUES
 *	The OP_VLS of the link to be used.
 *
@@ -1456,5 +1483,92 @@ osm_physp_replace_dr_path_with_alternate_dr_path(IN osm_log_t * p_log,
 *	PhysPort object
 *********/
 
+/****s* OpenSM: Port/osm_alias_guid_t
+* NAME
+*	osm_alias_guid_t
+*
+* DESCRIPTION
+*	This object represents an alias guid for an endport.
+*
+*	The osm_alias_guid_t object should be treated as opaque and should
+*	be manipulated only through the provided functions.
+*
+* SYNOPSIS
+*/
+typedef struct osm_alias_guid {
+	cl_map_item_t map_item;
+	ib_net64_t alias_guid;
+	osm_port_t *p_base_port;
+} osm_alias_guid_t;
+/*
+* FIELDS
+*	map_item
+*		Linkage structure for cl_qmap.  MUST BE FIRST MEMBER!
+*
+*	alias_guid
+*		Alias GUID for port obtained from SM GUIDInfo attribute
+*
+*	p_base_port
+*		Pointer to osm_port_t for base port GUID
+*
+* SEE ALSO
+*	Port, Physical Port, Physical Port Table
+*/
+
+/****f* OpenSM: Port/osm_alias_guid_new
+* NAME
+*	osm_alias_guid_new
+*
+* DESCRIPTION
+*	This function allocates and initializes an alias guid object.
+*
+* SYNOPSIS
+*/
+osm_alias_guid_t *osm_alias_guid_new(IN const ib_net64_t alias_guid,
+				     IN osm_port_t *p_base_port);
+/*
+* PARAMETERS
+*	alias_guid
+*		[in] Alias GUID in network order
+*
+*	p_base_port
+*		[in] Pointer to the port for this base GUID
+*
+* RETURN VALUE
+*	Pointer to the initialized alias guid object.
+*
+* NOTES
+*	Allows calling other alias guid methods.
+*
+* SEE ALSO
+*	Port
+*********/
+
+/****f* OpenSM: Port/osm_alias_guid_delete
+* NAME
+*	osm_alias_guid_delete
+*
+* DESCRIPTION
+*	This function destroys and deallocates an alias guid object.
+*
+* SYNOPSIS
+*/
+void osm_alias_guid_delete(IN OUT osm_alias_guid_t ** pp_alias_guid);
+/*
+* PARAMETERS
+*	pp_alias_guid
+*		[in][out] Pointer to a pointer to an alias guid object to delete.
+*		On return, this pointer is NULL.
+*
+* RETURN VALUE
+*	This function does not return a value.
+*
+* NOTES
+*	Performs any necessary cleanup of the specified alias guid object.
+*
+* SEE ALSO
+*	Port
+*********/
+
 END_C_DECLS
 #endif				/* _OSM_PORT_H_ */
