@@ -61,6 +61,8 @@
 #include <complib/cl_qlist.h>
 #include <complib/cl_math.h>
 #include <complib/cl_debug.h>
+#include <opensm/osm_file_ids.h>
+#define FILE_ID OSM_FILE_VENDOR_IBUMAD_C
 #include <opensm/osm_madw.h>
 #include <opensm/osm_log.h>
 #include <opensm/osm_mad_pool.h>
@@ -98,6 +100,35 @@ typedef struct _umad_receiver {
 
 static void osm_vendor_close_port(osm_vendor_t * const p_vend);
 
+static void log_send_error(osm_vendor_t * const p_vend, osm_madw_t *p_madw)
+{
+	if (p_madw->p_mad->mgmt_class != IB_MCLASS_SUBN_DIR) {
+		/* LID routed */
+		OSM_LOG(p_vend->p_log, OSM_LOG_ERROR, "ERR 5410: "
+			"Send completed with error (%s) -- dropping\n"
+			"\t\t\tClass 0x%x, Method 0x%X, Attr 0x%X, "
+			"TID 0x%" PRIx64 ", LID %u\n",
+			ib_get_err_str(p_madw->status),
+			p_madw->p_mad->mgmt_class, p_madw->p_mad->method,
+			cl_ntoh16(p_madw->p_mad->attr_id),
+			cl_ntoh64(p_madw->p_mad->trans_id),
+			cl_ntoh16(p_madw->mad_addr.dest_lid));
+	} else {
+		ib_smp_t *p_smp;
+
+		/* Direct routed SMP */
+		p_smp = osm_madw_get_smp_ptr(p_madw);
+		OSM_LOG(p_vend->p_log, OSM_LOG_ERROR, "ERR 5411: "
+			"DR SMP Send completed with error (%s) -- dropping\n"
+			"\t\t\tMethod 0x%X, Attr 0x%X, TID 0x%" PRIx64 "\n",
+			ib_get_err_str(p_madw->status),
+			p_madw->p_mad->method,
+			cl_ntoh16(p_madw->p_mad->attr_id),
+			cl_ntoh64(p_madw->p_mad->trans_id));
+		osm_dump_smp_dr_path(p_vend->p_log, p_smp, OSM_LOG_ERROR);
+	}
+}
+
 static void clear_madw(osm_vendor_t * p_vend)
 {
 	umad_match_t *m, *e, *old_m;
@@ -126,7 +157,8 @@ Exit:
 	OSM_LOG_EXIT(p_vend->p_log);
 }
 
-static osm_madw_t *get_madw(osm_vendor_t * p_vend, ib_net64_t * tid)
+static osm_madw_t *get_madw(osm_vendor_t * p_vend, ib_net64_t * tid,
+			    uint8_t mgmt_class)
 {
 	umad_match_t *m, *e;
 	ib_net64_t mtid = (*tid & CL_HTON64(0x00000000ffffffffULL));
@@ -136,13 +168,14 @@ static osm_madw_t *get_madw(osm_vendor_t * p_vend, ib_net64_t * tid)
 	 * Since mtid == 0 is the empty key, we should not
 	 * waste time looking for it
 	 */
-	if (mtid == 0)
+	if (mtid == 0 || mgmt_class == 0)
 		return 0;
 
 	pthread_mutex_lock(&p_vend->match_tbl_mutex);
 	for (m = p_vend->mtbl.tbl, e = m + p_vend->mtbl.max; m < e; m++) {
-		if (m->tid == mtid) {
+		if (m->tid == mtid && m->mgmt_class == mgmt_class) {
 			m->tid = 0;
+			m->mgmt_class = 0;
 			*tid = mtid;
 			res = m->v;
 			pthread_mutex_unlock(&p_vend->match_tbl_mutex);
@@ -155,18 +188,21 @@ static osm_madw_t *get_madw(osm_vendor_t * p_vend, ib_net64_t * tid)
 }
 
 static void
-put_madw(osm_vendor_t * p_vend, osm_madw_t * p_madw, ib_net64_t tid)
+put_madw(osm_vendor_t * p_vend, osm_madw_t * p_madw, ib_net64_t tid,
+	 uint8_t mgmt_class)
 {
 	umad_match_t *m, *e, *old_lru, *lru = 0;
 	osm_madw_t *p_req_madw;
 	osm_umad_bind_info_t *p_bind;
 	ib_net64_t old_tid;
 	uint32_t oldest = ~0;
+	uint8_t old_mgmt_class;
 
 	pthread_mutex_lock(&p_vend->match_tbl_mutex);
 	for (m = p_vend->mtbl.tbl, e = m + p_vend->mtbl.max; m < e; m++) {
-		if (m->tid == 0) {
+		if (m->tid == 0 && m->mgmt_class == 0) {
 			m->tid = tid;
+			m->mgmt_class = mgmt_class;
 			m->v = p_madw;
 			m->version =
 			    cl_atomic_inc((atomic32_t *) & p_vend->mtbl.
@@ -174,7 +210,7 @@ put_madw(osm_vendor_t * p_vend, osm_madw_t * p_madw, ib_net64_t tid)
 			pthread_mutex_unlock(&p_vend->match_tbl_mutex);
 			return;
 		}
-		if (oldest > m->version) {
+		if (oldest >= m->version) {
 			oldest = m->version;
 			lru = m;
 		}
@@ -182,20 +218,24 @@ put_madw(osm_vendor_t * p_vend, osm_madw_t * p_madw, ib_net64_t tid)
 
 	old_lru = lru;
 	old_tid = lru->tid;
+	old_mgmt_class = lru->mgmt_class;
 	p_req_madw = old_lru->v;
 	p_bind = p_req_madw->h_bind;
 	p_req_madw->status = IB_CANCELED;
+	log_send_error(p_vend, p_req_madw);
 	pthread_mutex_lock(&p_vend->cb_mutex);
 	(*p_bind->send_err_callback) (p_bind->client_context, p_req_madw);
 	pthread_mutex_unlock(&p_vend->cb_mutex);
 	lru->tid = tid;
+	lru->mgmt_class = mgmt_class;
 	lru->v = p_madw;
 	lru->version =
 	    cl_atomic_inc((atomic32_t *) & p_vend->mtbl.last_version);
 	pthread_mutex_unlock(&p_vend->match_tbl_mutex);
 	OSM_LOG(p_vend->p_log, OSM_LOG_ERROR, "ERR 5402: "
-		"evicting entry %p (tid was 0x%" PRIx64 ")\n", old_lru,
-		cl_ntoh64(old_tid));
+		"evicting entry %p (tid was 0x%" PRIx64
+		" mgmt class 0x%x)\n", old_lru,
+		cl_ntoh64(old_tid), old_mgmt_class);
 }
 
 static void
@@ -243,7 +283,6 @@ static void *umad_receiver(void *p_ptr)
 	umad_receiver_t *const p_ur = (umad_receiver_t *) p_ptr;
 	osm_vendor_t *p_vend = p_ur->p_vend;
 	osm_umad_bind_info_t *p_bind;
-	ib_mad_addr_t *ib_mad_addr;
 	osm_mad_addr_t osm_addr;
 	osm_madw_t *p_madw, *p_req_madw;
 	ib_mad_t *mad;
@@ -299,7 +338,6 @@ static void *umad_receiver(void *p_ptr)
 		}
 
 		mad = (ib_mad_t *) umad_get_mad(umad);
-		ib_mad_addr = umad_get_mad_addr(umad);
 
 		ib_mad_addr_conv(umad, &osm_addr,
 				 mad->mgmt_class == IB_MCLASS_SUBN_LID ||
@@ -326,41 +364,18 @@ static void *umad_receiver(void *p_ptr)
 
 		/* if status != 0 then we are handling recv timeout on send */
 		if (umad_status(p_madw->vend_wrap.umad)) {
-
-			if (mad->mgmt_class != IB_MCLASS_SUBN_DIR) {
-				/* LID routed */
-				OSM_LOG(p_vend->p_log, OSM_LOG_ERROR, "ERR 5410: "
-					"Send completed with error -- dropping\n"
-					"\t\t\tClass 0x%x, Method 0x%X, Attr 0x%X, "
-					"TID 0x%" PRIx64 ", LID %u\n",
-					mad->mgmt_class, mad->method,
-					cl_ntoh16(mad->attr_id),
-					cl_ntoh64(mad->trans_id),
-					cl_ntoh16(ib_mad_addr->lid));
-			} else {
-				ib_smp_t *smp;
-
-				/* Direct routed SMP */
-				smp = (ib_smp_t *) mad;
-				OSM_LOG(p_vend->p_log, OSM_LOG_ERROR, "ERR 5411: "
-					"DR SMP Send completed with error -- dropping\n"
-					"\t\t\tMethod 0x%X, Attr 0x%X, TID 0x%" PRIx64
-					", Hop Ptr: 0x%X\n",
-					mad->method, cl_ntoh16(mad->attr_id),
-					cl_ntoh64(mad->trans_id), smp->hop_ptr);
-				osm_dump_smp_dr_path(p_vend->p_log, smp,
-						     OSM_LOG_ERROR);
-			}
-
-			if (!(p_req_madw = get_madw(p_vend, &mad->trans_id))) {
+			if (!(p_req_madw = get_madw(p_vend, &mad->trans_id,
+						    mad->mgmt_class))) {
 				OSM_LOG(p_vend->p_log, OSM_LOG_ERROR,
 					"ERR 5412: "
 					"Failed to obtain request madw for timed out MAD"
-					" (method=0x%X attr=0x%X tid=0x%"PRIx64") -- dropping\n",
-					mad->method, cl_ntoh16(mad->attr_id),
+					" (class=0x%X method=0x%X attr=0x%X tid=0x%"PRIx64") -- dropping\n",
+					mad->mgmt_class, mad->method,
+					cl_ntoh16(mad->attr_id),
 					cl_ntoh64(mad->trans_id));
 			} else {
 				p_req_madw->status = IB_TIMEOUT;
+				log_send_error(p_vend, p_req_madw);
 				/* cb frees req_madw */
 				pthread_mutex_lock(&p_vend->cb_mutex);
 				pthread_cleanup_push(unlock_mutex,
@@ -377,11 +392,13 @@ static void *umad_receiver(void *p_ptr)
 
 		p_req_madw = 0;
 		if (ib_mad_is_response(mad) &&
-		    !(p_req_madw = get_madw(p_vend, &mad->trans_id))) {
+		    !(p_req_madw = get_madw(p_vend, &mad->trans_id,
+					    mad->mgmt_class))) {
 			OSM_LOG(p_vend->p_log, OSM_LOG_ERROR, "ERR 5413: "
 				"Failed to obtain request madw for received MAD"
-				"(method=0x%X attr=0x%X tid=0x%"PRIx64") -- dropping\n",
-				mad->method, cl_ntoh16((mad)->attr_id),
+				" (class=0x%X method=0x%X attr=0x%X tid=0x%"PRIx64") -- dropping\n",
+				mad->mgmt_class, mad->method,
+				cl_ntoh16((mad)->attr_id),
 				cl_ntoh64(mad->trans_id));
 			osm_mad_pool_put(p_bind->p_mad_pool, p_madw);
 			continue;
@@ -422,7 +439,7 @@ static int umad_receiver_start(osm_vendor_t * p_vend)
 	p_ur->p_vend = p_vend;
 	p_ur->p_log = p_vend->p_log;
 
-	if (pthread_create(&p_ur->tid, NULL, umad_receiver, p_ur) < 0)
+	if (pthread_create(&p_ur->tid, NULL, umad_receiver, p_ur) != 0)
 		return -1;
 
 	return 0;
@@ -637,9 +654,8 @@ osm_vendor_open_port(IN osm_vendor_t * const p_vend,
 	}
 
 	for (ca = 0; ca < p_vend->ca_count; ca++) {
-		if ((r = umad_get_ca_portguids(p_vend->ca_names[ca],
-					       portguids,
-					       OSM_UMAD_MAX_CAS)) < 0) {
+		if ((r = umad_get_ca_portguids(p_vend->ca_names[ca], portguids,
+					       OSM_UMAD_MAX_PORTS_PER_CA + 1)) < 0) {
 			OSM_LOG(p_vend->p_log, OSM_LOG_ERROR, "ERR 5421: "
 				"Unable to get CA %s port guids (%s)\n",
 				p_vend->ca_names[ca], strerror(r));
@@ -779,7 +795,8 @@ osm_vendor_bind(IN osm_vendor_t * const p_vend,
 	port_guid = p_user_bind->port_guid;
 
 	OSM_LOG(p_vend->p_log, OSM_LOG_INFO,
-		"Binding to port 0x%" PRIx64 "\n", cl_ntoh64(port_guid));
+		"Mgmt class 0x%02x binding to port GUID 0x%" PRIx64 "\n",
+		p_user_bind->mad_class, cl_ntoh64(port_guid));
 
 	if ((umad_port_id = osm_vendor_open_port(p_vend, port_guid)) < 0) {
 		OSM_LOG(p_vend->p_log, OSM_LOG_ERROR, "ERR 5424: "
@@ -1003,8 +1020,9 @@ osm_vendor_send(IN osm_bind_handle_t h_bind,
 	ib_mad_t *const p_mad = osm_madw_get_mad_ptr(p_madw);
 	ib_sa_mad_t *const p_sa = (ib_sa_mad_t *) p_mad;
 	int ret = -1;
-	int is_rmpp = 0;
+	int __attribute__((__unused__)) is_rmpp = 0;
 	uint32_t sent_mad_size;
+	uint64_t tid;
 #ifndef VENDOR_RMPP_SUPPORT
 	uint32_t paylen = 0;
 #endif
@@ -1042,10 +1060,10 @@ osm_vendor_send(IN osm_bind_handle_t h_bind,
 #ifdef VENDOR_RMPP_SUPPORT
 		} else
 			is_rmpp = 1;
-			OSM_LOG(p_vend->p_log, OSM_LOG_VERBOSE, "RMPP %d length %d\n",
-				ib_rmpp_is_flag_set((ib_rmpp_mad_t *) p_sa,
-						    IB_RMPP_FLAG_ACTIVE),
-				p_madw->mad_size);
+		OSM_LOG(p_vend->p_log, OSM_LOG_DEBUG, "RMPP %d length %d\n",
+			ib_rmpp_is_flag_set((ib_rmpp_mad_t *) p_sa,
+					    IB_RMPP_FLAG_ACTIVE),
+			p_madw->mad_size);
 #else
 		} else {
 			p_sa->rmpp_version = 1;
@@ -1061,7 +1079,7 @@ osm_vendor_send(IN osm_bind_handle_t h_bind,
 
 Resp:
 	if (resp_expected)
-		put_madw(p_vend, p_madw, p_mad->trans_id);
+		put_madw(p_vend, p_madw, p_mad->trans_id, p_mad->mgmt_class);
 
 #ifdef VENDOR_RMPP_SUPPORT
 	sent_mad_size = p_madw->mad_size;
@@ -1069,15 +1087,17 @@ Resp:
 	sent_mad_size = is_rmpp ? p_madw->mad_size - IB_SA_MAD_HDR_SIZE :
 	    p_madw->mad_size;
 #endif
+	tid = cl_ntoh64(p_mad->trans_id);
 	if ((ret = umad_send(p_bind->port_id, p_bind->agent_id, p_vw->umad,
 			     sent_mad_size,
 			     resp_expected ? p_bind->timeout : 0,
 			     p_bind->max_retries)) < 0) {
 		OSM_LOG(p_vend->p_log, OSM_LOG_ERROR, "ERR 5430: "
 			"Send p_madw = %p of size %d TID 0x%" PRIx64 " failed %d (%m)\n",
-			p_madw, sent_mad_size, cl_ntoh64(p_mad->trans_id), ret);
+			p_madw, sent_mad_size, tid, ret);
 		if (resp_expected) {
-			get_madw(p_vend, &p_mad->trans_id);	/* remove from aging table */
+			get_madw(p_vend, &p_mad->trans_id,
+				 p_mad->mgmt_class);	/* remove from aging table */
 			p_madw->status = IB_ERROR;
 			pthread_mutex_lock(&p_vend->cb_mutex);
 			(*p_bind->send_err_callback) (p_bind->client_context, p_madw);	/* cb frees madw */
@@ -1090,8 +1110,8 @@ Resp:
 	if (!resp_expected)
 		osm_mad_pool_put(p_bind->p_mad_pool, p_madw);
 
-	OSM_LOG(p_vend->p_log, OSM_LOG_DEBUG, "Completed sending %s p_madw = %p\n",
-		resp_expected ? "request" : "response or unsolicited", p_madw);
+	OSM_LOG(p_vend->p_log, OSM_LOG_DEBUG, "Completed sending %s TID 0x%" PRIx64 "\n",
+		resp_expected ? "request" : "response or unsolicited", tid);
 Exit:
 	OSM_LOG_EXIT(p_vend->p_log);
 	return (ret);
