@@ -46,8 +46,11 @@
 #include <sys/types.h>
 #include <stdlib.h>
 #include <string.h>
+#include <opensm/osm_file_ids.h>
+#define FILE_ID OSM_FILE_DB_FILES_C
 #include <opensm/st.h>
 #include <opensm/osm_db.h>
+#include <opensm/osm_log.h>
 
 /****d* Database/OSM_DB_MAX_LINE_LEN
  * NAME
@@ -157,7 +160,10 @@ int osm_db_init(IN osm_db_t * p_db, IN osm_log_t * p_log)
 	OSM_LOG_ENTER(p_log);
 
 	p_db_imp = malloc(sizeof(osm_db_imp_t));
-	CL_ASSERT(p_db_imp != NULL);
+	if (!p_db_imp) {
+		OSM_LOG(p_log, OSM_LOG_ERROR, "ERR 6100: No memory.\n");
+		return -1;
+	}
 
 	p_db_imp->db_dir_name = getenv("OSM_CACHE_DIR");
 	if (!p_db_imp->db_dir_name || !(*p_db_imp->db_dir_name))
@@ -166,18 +172,19 @@ int osm_db_init(IN osm_db_t * p_db, IN osm_log_t * p_log)
 	/* Create the directory if it doesn't exist */
 	/* There is a difference in creating directory between windows and linux */
 #ifdef __WIN__
-	/* Check if the directory exists. If not - create it. */
-	CreateDirectory(p_db_imp->db_dir_name, NULL);
+	{
+		int ret;
+
+		ret = SHCreateDirectoryEx(NULL, p_db_imp->db_dir_name, NULL);
+		if (ret != ERROR_SUCCESS && ret != ERROR_ALREADY_EXISTS &&
+			ret != ERROR_FILE_EXISTS)
+			goto err;
+	}
 #else				/* __WIN__ */
 	/* make sure the directory exists */
 	if (lstat(p_db_imp->db_dir_name, &dstat)) {
-		if (mkdir(p_db_imp->db_dir_name, 0755)) {
-			OSM_LOG(p_log, OSM_LOG_ERROR, "ERR 6101: "
-				"Failed to create the db directory:%s\n",
-				p_db_imp->db_dir_name);
-			OSM_LOG_EXIT(p_log);
-			return 1;
-		}
+		if (mkdir(p_db_imp->db_dir_name, 0755))
+			goto err;
 	}
 #endif
 
@@ -189,6 +196,14 @@ int osm_db_init(IN osm_db_t * p_db, IN osm_log_t * p_log)
 	OSM_LOG_EXIT(p_log);
 
 	return 0;
+
+err:
+	OSM_LOG(p_log, OSM_LOG_ERROR, "ERR 6101: "
+		"Failed to create the db directory:%s\n",
+		p_db_imp->db_dir_name);
+	free(p_db_imp);
+	OSM_LOG_EXIT(p_log);
+	return 1;
 }
 
 osm_db_domain_t *osm_db_domain_init(IN osm_db_t * p_db, IN char *domain_name)
@@ -582,7 +597,6 @@ int osm_db_delete(IN osm_db_domain_t * p_domain, IN char *p_key)
 				p_key, p_domain_imp->file_name, p_prev_val);
 			res = 1;
 		} else {
-			free(p_key);
 			free(p_prev_val);
 			res = 0;
 		}
