@@ -85,6 +85,8 @@
 #include <iba/ib_types.h>
 #include <complib/cl_qmap.h>
 #include <complib/cl_debug.h>
+#include <opensm/osm_file_ids.h>
+#define FILE_ID OSM_FILE_LID_MGR_C
 #include <opensm/osm_lid_mgr.h>
 #include <opensm/osm_sm.h>
 #include <opensm/osm_log.h>
@@ -115,11 +117,9 @@ void osm_lid_mgr_destroy(IN osm_lid_mgr_t * p_mgr)
 
 	OSM_LOG_ENTER(p_mgr->p_log);
 
-	p_item = cl_qlist_remove_head(&p_mgr->free_ranges);
-	while (p_item != cl_qlist_end(&p_mgr->free_ranges)) {
+	while ((p_item = cl_qlist_remove_head(&p_mgr->free_ranges)) !=
+	       cl_qlist_end(&p_mgr->free_ranges))
 		free((osm_lid_mgr_range_t *) p_item);
-		p_item = cl_qlist_remove_head(&p_mgr->free_ranges);
-	}
 	OSM_LOG_EXIT(p_mgr->p_log);
 }
 
@@ -150,8 +150,8 @@ static void lid_mgr_validate_db(IN osm_lid_mgr_t * p_mgr)
 		goto Exit;
 	}
 
-	p_item = (osm_db_guid_elem_t *) cl_qlist_remove_head(&guids);
-	while ((cl_list_item_t *) p_item != cl_qlist_end(&guids)) {
+	while ((p_item = (osm_db_guid_elem_t *) cl_qlist_remove_head(&guids))
+	       != (osm_db_guid_elem_t *) cl_qlist_end(&guids)) {
 		if (osm_db_guid2lid_get(p_mgr->p_g2l, p_item->guid,
 					&min_lid, &max_lid))
 			OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR, "ERR 0311: "
@@ -197,22 +197,18 @@ static void lid_mgr_validate_db(IN osm_lid_mgr_t * p_mgr)
 				}
 			}
 
-			if (!lids_ok) {
-				if (osm_db_guid2lid_delete(p_mgr->p_g2l,
-							   p_item->guid))
-					OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR,
-						"ERR 0315: "
-						"failed to delete entry for "
-						"guid:0x%016" PRIx64 "\n",
-						p_item->guid);
-			} else {
+			if (lids_ok)
 				/* mark that it was visited */
 				for (lid = min_lid; lid <= max_lid; lid++)
 					p_mgr->used_lids[lid] = 1;
-			}
+			else if (osm_db_guid2lid_delete(p_mgr->p_g2l,
+							p_item->guid))
+				OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR,
+					"ERR 0315: failed to delete entry for "
+					"guid:0x%016" PRIx64 "\n",
+					p_item->guid);
 		}		/* got a lid */
 		free(p_item);
-		p_item = (osm_db_guid_elem_t *) cl_qlist_remove_head(&guids);
 	}			/* all guids */
 Exit:
 	OSM_LOG_EXIT(p_mgr->p_log);
@@ -253,9 +249,9 @@ ib_api_status_t osm_lid_mgr_init(IN osm_lid_mgr_t * p_mgr, IN osm_sm_t * sm)
 			 * are closed, so we might see corrupted guid2lid file.
 			 */
 			if (p_mgr->p_subn->opt.exit_on_fatal) {
-				osm_log(p_mgr->p_log, OSM_LOG_SYS,
-					"FATAL: Error restoring Guid-to-Lid "
-					"persistent database\n");
+				osm_log_v2(p_mgr->p_log, OSM_LOG_SYS, FILE_ID,
+					   "FATAL: Error restoring Guid-to-Lid "
+					   "persistent database\n");
 				status = IB_ERROR;
 				goto Exit;
 			} else
@@ -291,14 +287,8 @@ static uint16_t trim_lid(IN uint16_t lid)
 static int lid_mgr_init_sweep(IN osm_lid_mgr_t * p_mgr)
 {
 	cl_ptr_vector_t *p_discovered_vec = &p_mgr->p_subn->port_lid_tbl;
-	uint16_t max_defined_lid;
-	uint16_t max_persistent_lid;
-	uint16_t max_discovered_lid;
-	uint16_t lid;
-	uint16_t disc_min_lid;
-	uint16_t disc_max_lid;
-	uint16_t db_min_lid;
-	uint16_t db_max_lid;
+	uint16_t max_defined_lid, max_persistent_lid, max_discovered_lid;
+	uint16_t disc_min_lid, disc_max_lid, db_min_lid, db_max_lid;
 	int status = 0;
 	cl_list_item_t *p_item;
 	boolean_t is_free;
@@ -306,8 +296,7 @@ static int lid_mgr_init_sweep(IN osm_lid_mgr_t * p_mgr)
 	osm_port_t *p_port;
 	cl_qmap_t *p_port_guid_tbl;
 	uint8_t lmc_num_lids = (uint8_t) (1 << p_mgr->p_subn->opt.lmc);
-	uint16_t lmc_mask;
-	uint16_t req_lid, num_lids;
+	uint16_t lmc_mask, req_lid, num_lids, lid;
 
 	OSM_LOG_ENTER(p_mgr->p_log);
 
@@ -318,30 +307,28 @@ static int lid_mgr_init_sweep(IN osm_lid_mgr_t * p_mgr)
 	   Do this only if the honor_guid2lid_file option is FALSE. If not, then
 	   need to honor this file. */
 	if (p_mgr->p_subn->coming_out_of_standby == TRUE) {
-		if (p_mgr->p_subn->opt.honor_guid2lid_file == FALSE) {
+		osm_db_clear(p_mgr->p_g2l);
+		memset(p_mgr->used_lids, 0, sizeof(p_mgr->used_lids));
+		if (p_mgr->p_subn->opt.honor_guid2lid_file == FALSE)
 			OSM_LOG(p_mgr->p_log, OSM_LOG_DEBUG,
 				"Ignore guid2lid file when coming out of standby\n");
-			osm_db_clear(p_mgr->p_g2l);
-			memset(p_mgr->used_lids, 0, sizeof(p_mgr->used_lids));
-		} else {
+		else {
 			OSM_LOG(p_mgr->p_log, OSM_LOG_DEBUG,
 				"Honor current guid2lid file when coming out "
 				"of standby\n");
-			osm_db_clear(p_mgr->p_g2l);
 			if (osm_db_restore(p_mgr->p_g2l))
 				OSM_LOG(p_mgr->p_log, OSM_LOG_ERROR,
 					"ERR 0306: "
 					"Error restoring Guid-to-Lid "
 					"persistent database. Ignoring it\n");
+			lid_mgr_validate_db(p_mgr);
 		}
 	}
 
 	/* we need to cleanup the empty ranges list */
-	p_item = cl_qlist_remove_head(&p_mgr->free_ranges);
-	while (p_item != cl_qlist_end(&p_mgr->free_ranges)) {
+	while ((p_item = cl_qlist_remove_head(&p_mgr->free_ranges)) !=
+	       cl_qlist_end(&p_mgr->free_ranges))
 		free((osm_lid_mgr_range_t *) p_item);
-		p_item = cl_qlist_remove_head(&p_mgr->free_ranges);
-	}
 
 	/* first clean up the port_by_lid_tbl */
 	for (lid = 0; lid < cl_ptr_vector_get_size(p_discovered_vec); lid++)
@@ -354,8 +341,7 @@ static int lid_mgr_init_sweep(IN osm_lid_mgr_t * p_mgr)
 	    p_mgr->p_subn->opt.reassign_lids == TRUE) {
 		OSM_LOG(p_mgr->p_log, OSM_LOG_DEBUG,
 			"Skipping all lids as we are reassigning them\n");
-		p_range =
-		    (osm_lid_mgr_range_t *) malloc(sizeof(osm_lid_mgr_range_t));
+		p_range = malloc(sizeof(osm_lid_mgr_range_t));
 		if (p_range)
 			p_range->min_lid = 1;
 		goto AfterScanningLids;
@@ -442,10 +428,9 @@ static int lid_mgr_init_sweep(IN osm_lid_mgr_t * p_mgr)
 				"persistent db\n", lid);
 			is_free = FALSE;
 			/* check this is a discovered port */
-		} else if (lid <= max_discovered_lid && (p_port = (osm_port_t *)
-							 cl_ptr_vector_get
-							 (p_discovered_vec,
-							  lid))) {
+		} else if (lid <= max_discovered_lid &&
+			   (p_port = cl_ptr_vector_get(p_discovered_vec,
+						       lid))) {
 			/* we have a port. Now lets see if we can preserve its lid range. */
 			/* For that, we need to make sure:
 			   1. The port has a (legal) persistency entry. Then the
@@ -527,23 +512,20 @@ static int lid_mgr_init_sweep(IN osm_lid_mgr_t * p_mgr)
 			if (p_range)
 				p_range->max_lid = lid;
 			else {
-				p_range = (osm_lid_mgr_range_t *)
-				    malloc(sizeof(osm_lid_mgr_range_t));
+				p_range = malloc(sizeof(osm_lid_mgr_range_t));
 				if (p_range) {
 					p_range->min_lid = lid;
 					p_range->max_lid = lid;
 				}
 			}
-		} else {
-			/* this lid is used so we need to finalize the previous free range */
-			if (p_range) {
-				cl_qlist_insert_tail(&p_mgr->free_ranges,
-						     &p_range->item);
-				OSM_LOG(p_mgr->p_log, OSM_LOG_DEBUG,
-					"new free lid range [%u:%u]\n",
-					p_range->min_lid, p_range->max_lid);
-				p_range = NULL;
-			}
+		/* this lid is used so we need to finalize the previous free range */
+		} else if (p_range) {
+			cl_qlist_insert_tail(&p_mgr->free_ranges,
+					     &p_range->item);
+			OSM_LOG(p_mgr->p_log, OSM_LOG_DEBUG,
+				"new free lid range [%u:%u]\n",
+				p_range->min_lid, p_range->max_lid);
+			p_range = NULL;
 		}
 	}
 
@@ -551,8 +533,7 @@ AfterScanningLids:
 	/* after scanning all known lids we need to extend the last range
 	   to the max allowed lid */
 	if (!p_range) {
-		p_range =
-		    (osm_lid_mgr_range_t *) malloc(sizeof(osm_lid_mgr_range_t));
+		p_range = malloc(sizeof(osm_lid_mgr_range_t));
 		/*
 		   The p_range can be NULL in one of 2 cases:
 		   1. If max_defined_lid == 0. In this case, we want the
@@ -584,10 +565,6 @@ static boolean_t lid_mgr_is_range_not_persistent(IN osm_lid_mgr_t * p_mgr,
 						 IN uint16_t num_lids)
 {
 	uint16_t i;
-	uint8_t start_lid = (uint8_t) (1 << p_mgr->p_subn->opt.lmc);
-
-	if (lid < start_lid)
-		return FALSE;
 
 	for (i = lid; i < lid + num_lids; i++)
 		if (p_mgr->used_lids[i])
@@ -636,11 +613,12 @@ static void lid_mgr_find_free_lid_range(IN osm_lid_mgr_t * p_mgr,
 		/* but we can be out of the range */
 		if (lid + num_lids - 1 <= p_range->max_lid) {
 			/* ok let us use that range */
-			if (lid + num_lids - 1 == p_range->max_lid)
+			if (lid + num_lids - 1 == p_range->max_lid) {
 				/* we consumed the entire range */
 				cl_qlist_remove_item(&p_mgr->free_ranges,
 						     p_item);
-			else
+				free(p_item);
+			} else
 				/* only update the available range */
 				p_range->min_lid = lid + num_lids;
 
@@ -672,12 +650,10 @@ static void lid_mgr_cleanup_discovered_port_lid_range(IN osm_lid_mgr_t * p_mgr,
 	osm_port_get_lid_range_ho(p_port, &min_lid, &max_lid);
 	min_lid = trim_lid(min_lid);
 	max_lid = trim_lid(max_lid);
-	for (lid = min_lid; lid <= max_lid; lid++) {
-		if ((lid < max_tbl_lid) &&
-		    (p_port ==
-		     (osm_port_t *) cl_ptr_vector_get(p_discovered_vec, lid)))
+	for (lid = min_lid; lid <= max_lid; lid++)
+		if (lid < max_tbl_lid &&
+		    p_port == cl_ptr_vector_get(p_discovered_vec, lid))
 			cl_ptr_vector_set(p_discovered_vec, lid, NULL);
-	}
 }
 
 /**********************************************************************
@@ -719,11 +695,10 @@ static int lid_mgr_get_port_lid(IN osm_lid_mgr_t * p_mgr,
 	if (!osm_db_guid2lid_get(p_mgr->p_g2l, guid, &min_lid, &max_lid)) {
 		*p_min_lid = min_lid;
 		*p_max_lid = min_lid + num_lids - 1;
-		if (min_lid == cl_ntoh16(osm_port_get_base_lid(p_port))) {
+		if (min_lid == cl_ntoh16(osm_port_get_base_lid(p_port)))
 			OSM_LOG(p_mgr->p_log, OSM_LOG_DEBUG, "0x%016" PRIx64
 				" matches its known lid:%u\n", guid, min_lid);
-			goto Exit;
-		} else {
+		else {
 			OSM_LOG(p_mgr->p_log, OSM_LOG_DEBUG,
 				"0x%016" PRIx64 " with lid:%u "
 				"does not match its known lid:%u\n",
@@ -733,8 +708,8 @@ static int lid_mgr_get_port_lid(IN osm_lid_mgr_t * p_mgr,
 								  p_port);
 			/* we still need to send the setting to the target port */
 			lid_changed = 1;
-			goto Exit;
 		}
+		goto NewLidSet;
 	} else
 		OSM_LOG(p_mgr->p_log, OSM_LOG_DEBUG,
 			"0x%016" PRIx64 " has no persistent lid assigned\n",
@@ -788,7 +763,6 @@ NewLidSet:
 	for (lid = *p_min_lid; lid <= *p_max_lid; lid++)
 		p_mgr->used_lids[lid] = 1;
 
-Exit:
 	/* make sure the assigned lids are marked in port_lid_tbl */
 	for (lid = *p_min_lid; lid <= *p_max_lid; lid++)
 		cl_ptr_vector_set(&p_mgr->p_subn->port_lid_tbl, lid, p_port);
@@ -826,6 +800,7 @@ static int lid_mgr_set_physp_pi(IN osm_lid_mgr_t * p_mgr,
 	uint8_t op_vls;
 	uint8_t port_num;
 	boolean_t send_set = FALSE;
+	boolean_t update_mkey = FALSE;
 	int ret = 0;
 
 	OSM_LOG_ENTER(p_mgr->p_log);
@@ -866,8 +841,6 @@ static int lid_mgr_set_physp_pi(IN osm_lid_mgr_t * p_mgr,
 	 */
 
 	memcpy(payload, p_old_pi, sizeof(ib_port_info_t));
-	memset(payload + sizeof(ib_port_info_t), 0,
-	       IB_SMP_DATA_SIZE - sizeof(ib_port_info_t));
 
 	/*
 	   Should never write back a value that is bigger then 3 in
@@ -890,8 +863,10 @@ static int lid_mgr_set_physp_pi(IN osm_lid_mgr_t * p_mgr,
 		send_set = TRUE;
 
 	p_pi->m_key = p_mgr->p_subn->opt.m_key;
-	if (memcmp(&p_pi->m_key, &p_old_pi->m_key, sizeof(p_pi->m_key)))
+	if (memcmp(&p_pi->m_key, &p_old_pi->m_key, sizeof(p_pi->m_key))) {
+		update_mkey = TRUE;
 		send_set = TRUE;
+	}
 
 	p_pi->subnet_prefix = p_mgr->p_subn->opt.subnet_prefix;
 	if (memcmp(&p_pi->subnet_prefix, &p_old_pi->subnet_prefix,
@@ -914,6 +889,11 @@ static int lid_mgr_set_physp_pi(IN osm_lid_mgr_t * p_mgr,
 		   sizeof(p_pi->m_key_lease_period)))
 		send_set = TRUE;
 
+	p_pi->mkey_lmc = 0;
+	ib_port_info_set_mpb(p_pi, p_mgr->p_subn->opt.m_key_protect_bits);
+	if (ib_port_info_get_mpb(p_pi) != ib_port_info_get_mpb(p_old_pi))
+		send_set = TRUE;
+
 	/*
 	   we want to set the timeout for both the switch port 0
 	   and the CA ports
@@ -935,17 +915,18 @@ static int lid_mgr_set_physp_pi(IN osm_lid_mgr_t * p_mgr,
 			   sizeof(p_pi->link_width_enabled)))
 			send_set = TRUE;
 
-		/* M_KeyProtectBits are always zero */
-		p_pi->mkey_lmc = p_mgr->p_subn->opt.lmc;
-		if (memcmp(&p_pi->mkey_lmc, &p_old_pi->mkey_lmc,
-			   sizeof(p_pi->mkey_lmc)))
+		/* p_pi->mkey_lmc is initialized earlier */
+		ib_port_info_set_lmc(p_pi, p_mgr->p_subn->opt.lmc);
+		if (ib_port_info_get_lmc(p_pi) !=
+		    ib_port_info_get_lmc(p_old_pi))
 			send_set = TRUE;
 
 		/* calc new op_vls and mtu */
-		op_vls =
-		    osm_physp_calc_link_op_vls(p_mgr->p_log, p_mgr->p_subn,
-					       p_physp);
-		mtu = osm_physp_calc_link_mtu(p_mgr->p_log, p_physp);
+		op_vls = osm_physp_calc_link_op_vls(p_mgr->p_log, p_mgr->p_subn,
+					      p_physp,
+					      ib_port_info_get_op_vls(p_old_pi));
+		mtu = osm_physp_calc_link_mtu(p_mgr->p_log, p_physp,
+					      ib_port_info_get_neighbor_mtu(p_old_pi));
 
 		ib_port_info_set_neighbor_mtu(p_pi, mtu);
 
@@ -1018,10 +999,10 @@ static int lid_mgr_set_physp_pi(IN osm_lid_mgr_t * p_mgr,
 
 		/* Determine if enhanced switch port 0 and if so set LMC */
 		if (osm_switch_sp0_is_lmc_capable(p_node->sw, p_mgr->p_subn)) {
-			/* M_KeyProtectBits are always zero */
-			p_pi->mkey_lmc = p_mgr->p_subn->opt.lmc;
-			if (memcmp(&p_pi->mkey_lmc, &p_old_pi->mkey_lmc,
-				   sizeof(p_pi->mkey_lmc)))
+			/* p_pi->mkey_lmc is initialized earlier */
+			ib_port_info_set_lmc(p_pi, p_mgr->p_subn->opt.lmc);
+			if (ib_port_info_get_lmc(p_pi) !=
+			    ib_port_info_get_lmc(p_old_pi))
 				send_set = TRUE;
 		}
 	}
@@ -1039,7 +1020,9 @@ static int lid_mgr_set_physp_pi(IN osm_lid_mgr_t * p_mgr,
 	   the cli_rereg bit. We know that the port was just discovered if its
 	   is_new field is set.
 	 */
-	if ((p_mgr->p_subn->first_time_master_sweep == TRUE || p_port->is_new)
+	if ((p_mgr->p_subn->first_time_master_sweep == TRUE
+	     || p_mgr->p_subn->set_client_rereg_on_sweep == TRUE
+	     || p_port->is_new)
 	    && !p_mgr->p_subn->opt.no_clients_rereg
 	    && (p_old_pi->capability_mask & IB_PORT_CAP_HAS_CLIENT_REREG)) {
 		OSM_LOG(p_mgr->p_log, OSM_LOG_DEBUG,
@@ -1056,8 +1039,13 @@ static int lid_mgr_set_physp_pi(IN osm_lid_mgr_t * p_mgr,
 	   2. first_time_master_sweep flag on the subnet is TRUE. This means the
 	   SM just became master, and it then needs to send a PortInfo Set to
 	   every port.
+	   3. set_client_rereg_on_sweep is TRUE.  The one situation in which this
+	   is true but first_time_master_sweep is FALSE is when the SM receives
+	   a HANDOVER while in master.  We don't want to re-setup everything by
+	   setting first_time_master_sweep, but we do want to reset up this.
 	 */
-	if (p_mgr->p_subn->first_time_master_sweep == TRUE)
+	if (p_mgr->p_subn->first_time_master_sweep == TRUE
+	    || p_mgr->p_subn->set_client_rereg_on_sweep == TRUE)
 		send_set = TRUE;
 
 	if (!send_set)
@@ -1069,6 +1057,13 @@ static int lid_mgr_set_physp_pi(IN osm_lid_mgr_t * p_mgr,
 			     CL_DISP_MSGID_NONE, &context);
 	if (status != IB_SUCCESS)
 		ret = -1;
+	/* If we sent a new mkey above, update our guid2mkey map
+	   now, on the assumption that the SubnSet succeeds
+	*/
+	if (update_mkey)
+		osm_db_guid2mkey_set(p_mgr->p_subn->p_g2m,
+				     cl_ntoh64(p_physp->port_guid),
+				     cl_ntoh64(p_pi->m_key));
 
 Exit:
 	OSM_LOG_EXIT(p_mgr->p_log);
@@ -1212,8 +1207,9 @@ int osm_lid_mgr_process_subnet(IN osm_lid_mgr_t * p_mgr)
 		   to look for any field change and will only send an updated
 		   if required */
 		OSM_LOG(p_mgr->p_log, OSM_LOG_VERBOSE,
-			"Assigned port 0x%016" PRIx64 ", LID [%u,%u]\n",
-			cl_ntoh64(port_guid), min_lid_ho, max_lid_ho);
+			"Assigned port 0x%016" PRIx64 ", %s LID [%u,%u]\n",
+			cl_ntoh64(port_guid), lid_changed ? "new" : "",
+			min_lid_ho, max_lid_ho);
 
 		/* the proc returns the fact it sent a set port info */
 		if (lid_mgr_set_physp_pi(p_mgr, p_port, p_port->p_physp,
