@@ -45,6 +45,8 @@
 #include <dlfcn.h>
 #include <sys/stat.h>
 
+#include <opensm/osm_file_ids.h>
+#define FILE_ID OSM_FILE_PERFMGR_DB_C
 #include <opensm/osm_perfmgr_db.h>
 #include <opensm/osm_perfmgr.h>
 #include <opensm/osm_opensm.h>
@@ -103,9 +105,15 @@ static inline perfmgr_db_err_t bad_node_port(db_node_t * node, uint8_t port)
 		return PERFMGR_EVENT_DB_GUIDNOTFOUND;
 	if (port >= node->num_ports || (!node->esp0 && port == 0))
 		return PERFMGR_EVENT_DB_PORTNOTFOUND;
+
 	return PERFMGR_EVENT_DB_SUCCESS;
 }
 
+static inline void mark_port_valid(db_node_t * node, uint8_t port)
+{
+	node->ports[port].valid = TRUE;
+}
+
 /** =========================================================================
  */
 static db_node_t *malloc_node(uint64_t guid, boolean_t esp0,
@@ -129,8 +137,10 @@ static db_node_t *malloc_node(uint64_t guid, boolean_t esp0,
 		rc->ports[i].last_reset = cur_time;
 		rc->ports[i].err_previous.time = cur_time;
 		rc->ports[i].dc_previous.time = cur_time;
+		rc->ports[i].valid = FALSE;
 	}
 	snprintf(rc->node_name, sizeof(rc->node_name), "%s", name);
+	rc->active = FALSE;
 
 	return rc;
 
@@ -186,6 +196,75 @@ Exit:
 	return rc;
 }
 
+perfmgr_db_err_t
+perfmgr_db_delete_entry(perfmgr_db_t * db, uint64_t guid)
+{
+	cl_map_item_t * rc = cl_qmap_remove(&db->pc_data, guid);
+
+	if (rc == cl_qmap_end(&db->pc_data))
+		return(PERFMGR_EVENT_DB_GUIDNOTFOUND);
+
+	db_node_t *pc_node = (db_node_t *)rc;
+	free_node(pc_node);
+	return(PERFMGR_EVENT_DB_SUCCESS);
+}
+
+perfmgr_db_err_t
+perfmgr_db_delete_inactive(perfmgr_db_t * db, unsigned *cnt)
+{
+	perfmgr_db_err_t rc = PERFMGR_EVENT_DB_SUCCESS;
+	int i = 0;
+	int num = 0;
+	uint64_t * guid_list = NULL;
+	cl_map_item_t * p_map_item = cl_qmap_head(&db->pc_data);
+
+	if (p_map_item == cl_qmap_end(&db->pc_data)) {
+		rc = PERFMGR_EVENT_DB_SUCCESS;
+		goto Done;
+	}
+
+	while (p_map_item != cl_qmap_end(&db->pc_data)) {
+		db_node_t *n = (db_node_t *)p_map_item;
+		if (n->active == FALSE) {
+			guid_list = realloc(guid_list,
+					sizeof(*guid_list) * (num+1));
+			if (!guid_list) {
+				num = 0;
+				rc = PERFMGR_EVENT_DB_NOMEM;
+				goto Done;
+			}
+			guid_list[num] = n->node_guid;
+			num++;
+		}
+		p_map_item = cl_qmap_next(p_map_item);
+	}
+
+	for (i = 0 ; i < num; i++)
+		perfmgr_db_delete_entry(db, guid_list[i]);
+
+	free(guid_list);
+
+Done:
+	if (cnt)
+		*cnt = num;
+
+	return(rc);
+}
+
+perfmgr_db_err_t
+perfmgr_db_mark_active(perfmgr_db_t *db, uint64_t guid, boolean_t active)
+{
+	db_node_t *node = NULL;
+
+	cl_plock_excl_acquire(&db->lock);
+	node = get(db, guid);
+	if (node)
+		node->active = active;
+	cl_plock_release(&db->lock);
+	return (PERFMGR_EVENT_DB_SUCCESS);
+}
+
+
 /**********************************************************************
  * Dump a reading vs the previous reading to stdout
  **********************************************************************/
@@ -195,60 +274,60 @@ debug_dump_err_reading(perfmgr_db_t * db, uint64_t guid, uint8_t port_num,
 {
 	osm_log_t *log = db->perfmgr->log;
 
-	if (!osm_log_is_active(log, OSM_LOG_DEBUG))
+	if (!OSM_LOG_IS_ACTIVE_V2(log, OSM_LOG_DEBUG))
 		return;		/* optimize this a bit */
 
-	osm_log(log, OSM_LOG_DEBUG,
-		"GUID 0x%" PRIx64 " Port %u:\n", guid, port_num);
-	osm_log(log, OSM_LOG_DEBUG,
-		"sym %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
-		cur->symbol_err_cnt, port->err_previous.symbol_err_cnt,
-		port->err_total.symbol_err_cnt);
-	osm_log(log, OSM_LOG_DEBUG,
-		"ler %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
-		cur->link_err_recover, port->err_previous.link_err_recover,
-		port->err_total.link_err_recover);
-	osm_log(log, OSM_LOG_DEBUG,
-		"ld %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
-		cur->link_downed, port->err_previous.link_downed,
-		port->err_total.link_downed);
-	osm_log(log, OSM_LOG_DEBUG,
-		"re %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n", cur->rcv_err,
-		port->err_previous.rcv_err, port->err_total.rcv_err);
-	osm_log(log, OSM_LOG_DEBUG,
-		"rrp %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
-		cur->rcv_rem_phys_err, port->err_previous.rcv_rem_phys_err,
-		port->err_total.rcv_rem_phys_err);
-	osm_log(log, OSM_LOG_DEBUG,
-		"rsr %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
-		cur->rcv_switch_relay_err,
-		port->err_previous.rcv_switch_relay_err,
-		port->err_total.rcv_switch_relay_err);
-	osm_log(log, OSM_LOG_DEBUG,
-		"xd %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
-		cur->xmit_discards, port->err_previous.xmit_discards,
-		port->err_total.xmit_discards);
-	osm_log(log, OSM_LOG_DEBUG,
-		"xce %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
-		cur->xmit_constraint_err,
-		port->err_previous.xmit_constraint_err,
-		port->err_total.xmit_constraint_err);
-	osm_log(log, OSM_LOG_DEBUG,
-		"rce %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
-		cur->rcv_constraint_err, port->err_previous.rcv_constraint_err,
-		port->err_total.rcv_constraint_err);
-	osm_log(log, OSM_LOG_DEBUG,
-		"li %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
-		cur->link_integrity, port->err_previous.link_integrity,
-		port->err_total.link_integrity);
-	osm_log(log, OSM_LOG_DEBUG,
-		"bo %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
-		cur->buffer_overrun, port->err_previous.buffer_overrun,
-		port->err_total.buffer_overrun);
-	osm_log(log, OSM_LOG_DEBUG,
-		"vld %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
-		cur->vl15_dropped, port->err_previous.vl15_dropped,
-		port->err_total.vl15_dropped);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "GUID 0x%" PRIx64 " Port %u:\n", guid, port_num);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "sym %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
+		   cur->symbol_err_cnt, port->err_previous.symbol_err_cnt,
+		   port->err_total.symbol_err_cnt);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "ler %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
+		   cur->link_err_recover, port->err_previous.link_err_recover,
+		   port->err_total.link_err_recover);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "ld %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
+		   cur->link_downed, port->err_previous.link_downed,
+		   port->err_total.link_downed);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "re %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n", cur->rcv_err,
+		   port->err_previous.rcv_err, port->err_total.rcv_err);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "rrp %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
+		   cur->rcv_rem_phys_err, port->err_previous.rcv_rem_phys_err,
+		   port->err_total.rcv_rem_phys_err);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "rsr %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
+		   cur->rcv_switch_relay_err,
+		   port->err_previous.rcv_switch_relay_err,
+		   port->err_total.rcv_switch_relay_err);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "xd %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
+		   cur->xmit_discards, port->err_previous.xmit_discards,
+		   port->err_total.xmit_discards);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "xce %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
+		   cur->xmit_constraint_err,
+		   port->err_previous.xmit_constraint_err,
+		   port->err_total.xmit_constraint_err);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "rce %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
+		   cur->rcv_constraint_err, port->err_previous.rcv_constraint_err,
+		   port->err_total.rcv_constraint_err);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "li %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
+		   cur->link_integrity, port->err_previous.link_integrity,
+		   port->err_total.link_integrity);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "bo %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
+		   cur->buffer_overrun, port->err_previous.buffer_overrun,
+		   port->err_total.buffer_overrun);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "vld %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
+		   cur->vl15_dropped, port->err_previous.vl15_dropped,
+		   port->err_total.vl15_dropped);
 }
 
 /**********************************************************************
@@ -269,6 +348,8 @@ perfmgr_db_add_err_reading(perfmgr_db_t * db, uint64_t guid, uint8_t port,
 	if ((rc = bad_node_port(node, port)) != PERFMGR_EVENT_DB_SUCCESS)
 		goto Exit;
 
+	mark_port_valid(node, port);
+
 	p_port = &(node->ports[port]);
 	previous = &(node->ports[port].err_previous);
 
@@ -374,23 +455,23 @@ debug_dump_dc_reading(perfmgr_db_t * db, uint64_t guid, uint8_t port_num,
 		      db_port_t * port, perfmgr_db_data_cnt_reading_t * cur)
 {
 	osm_log_t *log = db->perfmgr->log;
-	if (!osm_log_is_active(log, OSM_LOG_DEBUG))
+	if (!OSM_LOG_IS_ACTIVE_V2(log, OSM_LOG_DEBUG))
 		return;
 
-	osm_log(log, OSM_LOG_DEBUG,
-		"xd %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
-		cur->xmit_data, port->dc_previous.xmit_data,
-		port->dc_total.xmit_data);
-	osm_log(log, OSM_LOG_DEBUG,
-		"rd %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n", cur->rcv_data,
-		port->dc_previous.rcv_data, port->dc_total.rcv_data);
-	osm_log(log, OSM_LOG_DEBUG,
-		"xp %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
-		cur->xmit_pkts, port->dc_previous.xmit_pkts,
-		port->dc_total.xmit_pkts);
-	osm_log(log, OSM_LOG_DEBUG,
-		"rp %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n", cur->rcv_pkts,
-		port->dc_previous.rcv_pkts, port->dc_total.rcv_pkts);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "xd %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
+		   cur->xmit_data, port->dc_previous.xmit_data,
+		   port->dc_total.xmit_data);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "rd %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n", cur->rcv_data,
+		   port->dc_previous.rcv_data, port->dc_total.rcv_data);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "xp %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n",
+		   cur->xmit_pkts, port->dc_previous.xmit_pkts,
+		   port->dc_total.xmit_pkts);
+	osm_log_v2(log, OSM_LOG_DEBUG, FILE_ID,
+		   "rp %" PRIu64 " <-- %" PRIu64 " (%" PRIu64 ")\n", cur->rcv_pkts,
+		   port->dc_previous.rcv_pkts, port->dc_total.rcv_pkts);
 }
 
 /**********************************************************************
@@ -411,6 +492,8 @@ perfmgr_db_add_dc_reading(perfmgr_db_t * db, uint64_t guid, uint8_t port,
 	if ((rc = bad_node_port(node, port)) != PERFMGR_EVENT_DB_SUCCESS)
 		goto Exit;
 
+	mark_port_valid(node, port);
+
 	p_port = &node->ports[port];
 	previous = &node->ports[port].dc_previous;
 
@@ -550,7 +633,7 @@ static void dump_node_mr(db_node_t * node, FILE * fp)
 {
 	int i = 0;
 
-	fprintf(fp, "\nName\tGUID\tPort\tLast Reset\t"
+	fprintf(fp, "\nName\tGUID\tActive\tPort\tLast Reset\t"
 		"%s\t%s\t"
 		"%s\t%s\t%s\t%s\t%s\t%s\t%s\t"
 		"%s\t%s\t%s\t%s\t%s\t%s\t%s\t"
@@ -577,16 +660,22 @@ static void dump_node_mr(db_node_t * node, FILE * fp)
 		"multicast_rcv_pkts");
 	for (i = (node->esp0) ? 0 : 1; i < node->num_ports; i++) {
 		char *since = ctime(&node->ports[i].last_reset);
+
+		if (!node->ports[i].valid)
+			continue;
+
 		since[strlen(since) - 1] = '\0';	/* remove \n */
 
 		fprintf(fp,
-			"%s\t0x%" PRIx64 "\t%d\t%s\t%" PRIu64 "\t%" PRIu64 "\t"
+			"%s\t0x%" PRIx64 "\t%s\t%d\t%s\t%" PRIu64 "\t%" PRIu64 "\t"
 			"%" PRIu64 "\t%" PRIu64 "\t%" PRIu64 "\t%" PRIu64 "\t"
 			"%" PRIu64 "\t%" PRIu64 "\t%" PRIu64 "\t" "%" PRIu64
 			"\t%" PRIu64 "\t%" PRIu64 "\t" "%" PRIu64 "\t%" PRIu64
 			"\t%" PRIu64 "\t%" PRIu64 "\t" "%" PRIu64 "\t%" PRIu64
 			"\t%" PRIu64 "\t%" PRIu64 "\n", node->node_name,
-			node->node_guid, i, since,
+			node->node_guid,
+			node->active ? "TRUE" : "FALSE",
+			i, since,
 			node->ports[i].err_total.symbol_err_cnt,
 			node->ports[i].err_total.link_err_recover,
 			node->ports[i].err_total.link_downed,
@@ -610,63 +699,163 @@ static void dump_node_mr(db_node_t * node, FILE * fp)
 	}
 }
 
+static void dump_hr_dc(FILE *fp, uint64_t val64, int data)
+{
+	char *unit = "";
+	uint64_t tmp = val64;
+	float val = 0.0;
+	int ui = 0;
+	int div = 1;
+
+	tmp /= 1024;
+	while (tmp) {
+		ui++;
+		tmp /= 1024;
+		div *= 1024;
+	}
+
+	val = (float)(val64);
+	if (data) {
+		val *= 4;
+		if (val/div > 1024) {
+			ui++;
+			div *= 1024;
+		}
+	}
+	val /= div;
+
+	switch (ui) {
+		case 1:
+			unit = "K";
+			break;
+		case 2:
+			unit = "M";
+			break;
+		case 3:
+			unit = "G";
+			break;
+		case 4:
+			unit = "T";
+			break;
+		case 5:
+			unit = "P";
+			break;
+		case 6:
+			unit = "E";
+			break;
+	}
+
+	fprintf(fp, " (%5.3f%s%s)\n", val, unit, data ? "B" : "");
+}
+
 /**********************************************************************
  * Output a human readable output of the port counters
  **********************************************************************/
-static void dump_node_hr(db_node_t * node, FILE * fp)
+static void dump_node_hr(db_node_t * node, FILE * fp, char *port, int err_only)
 {
-	int i = 0;
+	int i = (node->esp0) ? 0 : 1;
+	int num_ports = node->num_ports;
 
-	fprintf(fp, "\n");
-	for (i = (node->esp0) ? 0 : 1; i < node->num_ports; i++) {
+	if (port) {
+		char *end = NULL;
+		int p = strtoul(port, &end, 0);
+		if (port + strlen(port) == end && p >= i && p < num_ports) {
+			i = p;
+			num_ports = p+1;
+		} else {
+			fprintf(fp, "Warning: \"%s\" is not a valid port\n", port);
+		}
+	}
+	for (/* set above */; i < num_ports; i++) {
 		char *since = ctime(&node->ports[i].last_reset);
-		since[strlen(since) - 1] = '\0';	/* remove \n */
 
-		fprintf(fp, "\"%s\" 0x%" PRIx64 " port %d (Since %s)\n"
-			"     symbol_err_cnt       : %" PRIu64 "\n"
-			"     link_err_recover     : %" PRIu64 "\n"
-			"     link_downed          : %" PRIu64 "\n"
-			"     rcv_err              : %" PRIu64 "\n"
-			"     rcv_rem_phys_err     : %" PRIu64 "\n"
-			"     rcv_switch_relay_err : %" PRIu64 "\n"
-			"     xmit_discards        : %" PRIu64 "\n"
-			"     xmit_constraint_err  : %" PRIu64 "\n"
-			"     rcv_constraint_err   : %" PRIu64 "\n"
-			"     link_integrity_err   : %" PRIu64 "\n"
-			"     buf_overrun_err      : %" PRIu64 "\n"
-			"     vl15_dropped         : %" PRIu64 "\n"
-			"     xmit_data            : %" PRIu64 "\n"
-			"     rcv_data             : %" PRIu64 "\n"
-			"     xmit_pkts            : %" PRIu64 "\n"
-			"     rcv_pkts             : %" PRIu64 "\n"
-			"     unicast_xmit_pkts    : %" PRIu64 "\n"
-			"     unicast_rcv_pkts     : %" PRIu64 "\n"
-			"     multicast_xmit_pkts  : %" PRIu64 "\n"
-			"     multicast_rcv_pkts   : %" PRIu64 "\n",
-			node->node_name,
-			node->node_guid,
-			i,
-			since,
-			node->ports[i].err_total.symbol_err_cnt,
-			node->ports[i].err_total.link_err_recover,
-			node->ports[i].err_total.link_downed,
-			node->ports[i].err_total.rcv_err,
-			node->ports[i].err_total.rcv_rem_phys_err,
-			node->ports[i].err_total.rcv_switch_relay_err,
-			node->ports[i].err_total.xmit_discards,
-			node->ports[i].err_total.xmit_constraint_err,
-			node->ports[i].err_total.rcv_constraint_err,
-			node->ports[i].err_total.link_integrity,
-			node->ports[i].err_total.buffer_overrun,
-			node->ports[i].err_total.vl15_dropped,
-			node->ports[i].dc_total.xmit_data,
-			node->ports[i].dc_total.rcv_data,
-			node->ports[i].dc_total.xmit_pkts,
-			node->ports[i].dc_total.rcv_pkts,
-			node->ports[i].dc_total.unicast_xmit_pkts,
-			node->ports[i].dc_total.unicast_rcv_pkts,
-			node->ports[i].dc_total.multicast_xmit_pkts,
+		if (!node->ports[i].valid)
+			continue;
+
+		since[strlen(since) - 1] = '\0';	/* remove \n */
+		perfmgr_db_err_reading_t *err = &node->ports[i].err_total;
+
+		if (err_only
+		    && err->symbol_err_cnt == 0
+		    && err->link_err_recover == 0
+		    && err->link_downed == 0
+		    && err->rcv_err == 0
+		    && err->rcv_rem_phys_err == 0
+		    && err->rcv_switch_relay_err == 0
+		    && err->xmit_discards == 0
+		    && err->xmit_constraint_err == 0
+		    && err->rcv_constraint_err == 0
+		    && err->link_integrity == 0
+		    && err->buffer_overrun == 0
+		    && err->vl15_dropped == 0)
+			continue;
+
+		fprintf(fp, "\"%s\" 0x%" PRIx64 " active %s port %d (Since %s)\n",
+			node->node_name, node->node_guid,
+			node->active ? "TRUE":"FALSE", i, since);
+
+		if (!err_only || err->symbol_err_cnt != 0)
+			fprintf(fp, "     symbol_err_cnt       : %" PRIu64 "\n",
+				err->symbol_err_cnt);
+		if (!err_only || err->link_err_recover != 0)
+			fprintf(fp, "     link_err_recover     : %" PRIu64 "\n",
+				err->link_err_recover);
+		if (!err_only || err->link_downed != 0)
+			fprintf(fp, "     link_downed          : %" PRIu64 "\n",
+				err->link_downed);
+		if (!err_only || err->rcv_err != 0)
+			fprintf(fp, "     rcv_err              : %" PRIu64 "\n",
+				err->rcv_err);
+		if (!err_only || err->rcv_rem_phys_err != 0)
+			fprintf(fp, "     rcv_rem_phys_err     : %" PRIu64 "\n",
+				err->rcv_rem_phys_err);
+		if (!err_only || err->rcv_switch_relay_err != 0)
+			fprintf(fp, "     rcv_switch_relay_err : %" PRIu64 "\n",
+				err->rcv_switch_relay_err);
+		if (!err_only || err->xmit_discards != 0)
+			fprintf(fp, "     xmit_discards        : %" PRIu64 "\n",
+				err->xmit_discards);
+		if (!err_only || err->xmit_constraint_err != 0)
+			fprintf(fp, "     xmit_constraint_err  : %" PRIu64 "\n",
+				err->xmit_constraint_err);
+		if (!err_only || err->rcv_constraint_err != 0)
+			fprintf(fp, "     rcv_constraint_err   : %" PRIu64 "\n",
+				err->rcv_constraint_err);
+		if (!err_only || err->link_integrity != 0)
+			fprintf(fp, "     link_integrity_err   : %" PRIu64 "\n",
+				err->link_integrity);
+		if (!err_only || err->buffer_overrun != 0)
+			fprintf(fp, "     buf_overrun_err      : %" PRIu64 "\n",
+				err->buffer_overrun);
+		if (!err_only || err->vl15_dropped != 0)
+			fprintf(fp, "     vl15_dropped         : %" PRIu64 "\n",
+				err->vl15_dropped);
+
+		fprintf(fp, "     xmit_data            : %" PRIu64,
+			node->ports[i].dc_total.xmit_data);
+		dump_hr_dc(fp, node->ports[i].dc_total.xmit_data, 1);
+		fprintf(fp, "     rcv_data             : %" PRIu64,
+			node->ports[i].dc_total.rcv_data);
+		dump_hr_dc(fp, node->ports[i].dc_total.rcv_data, 1);
+		fprintf(fp, "     xmit_pkts            : %" PRIu64,
+			node->ports[i].dc_total.xmit_pkts);
+		dump_hr_dc(fp, node->ports[i].dc_total.xmit_pkts, 0);
+		fprintf(fp, "     rcv_pkts             : %" PRIu64,
+			node->ports[i].dc_total.rcv_pkts);
+		dump_hr_dc(fp, node->ports[i].dc_total.rcv_pkts, 0);
+		fprintf(fp, "     unicast_xmit_pkts    : %" PRIu64,
+			node->ports[i].dc_total.unicast_xmit_pkts);
+		dump_hr_dc(fp, node->ports[i].dc_total.unicast_xmit_pkts, 0);
+		fprintf(fp, "     unicast_rcv_pkts     : %" PRIu64,
+			node->ports[i].dc_total.unicast_rcv_pkts);
+		dump_hr_dc(fp, node->ports[i].dc_total.unicast_rcv_pkts, 0);
+		fprintf(fp, "     multicast_xmit_pkts  : %" PRIu64,
+			node->ports[i].dc_total.multicast_xmit_pkts);
+		dump_hr_dc(fp, node->ports[i].dc_total.multicast_xmit_pkts, 0);
+		fprintf(fp, "     multicast_rcv_pkts   : %" PRIu64,
 			node->ports[i].dc_total.multicast_rcv_pkts);
+		dump_hr_dc(fp, node->ports[i].dc_total.multicast_rcv_pkts, 0);
+
 	}
 }
 
@@ -688,16 +877,36 @@ static void db_dump(cl_map_item_t * const p_map_item, void *context)
 		break;
 	case PERFMGR_EVENT_DB_DUMP_HR:
 	default:
-		dump_node_hr(node, fp);
+		dump_node_hr(node, fp, NULL, 0);
 		break;
 	}
 }
 
+/**********************************************************************
+ * print all node data to fp
+ **********************************************************************/
+void
+perfmgr_db_print_all(perfmgr_db_t * db, FILE *fp, int err_only)
+{
+	cl_map_item_t *item;
+	db_node_t *node;
+
+	cl_plock_acquire(&db->lock);
+	item = cl_qmap_head(&db->pc_data);
+	while (item != cl_qmap_end(&db->pc_data)) {
+		node = (db_node_t *)item;
+		dump_node_hr(node, fp, NULL, err_only);
+		item = cl_qmap_next(item);
+	}
+	cl_plock_release(&db->lock);
+}
+
 /**********************************************************************
  * print node data to fp
  **********************************************************************/
 void
-perfmgr_db_print_by_name(perfmgr_db_t * db, char *nodename, FILE *fp)
+perfmgr_db_print_by_name(perfmgr_db_t * db, char *nodename, FILE *fp,
+			 char *port, int err_only)
 {
 	cl_map_item_t *item;
 	db_node_t *node;
@@ -709,7 +918,7 @@ perfmgr_db_print_by_name(perfmgr_db_t * db, char *nodename, FILE *fp)
 	while (item != cl_qmap_end(&db->pc_data)) {
 		node = (db_node_t *)item;
 		if (strcmp(node->node_name, nodename) == 0) {
-			dump_node_hr(node, fp);
+			dump_node_hr(node, fp, port, err_only);
 			goto done;
 		}
 		item = cl_qmap_next(item);
@@ -724,7 +933,8 @@ done:
  * print node data to fp
  **********************************************************************/
 void
-perfmgr_db_print_by_guid(perfmgr_db_t * db, uint64_t nodeguid, FILE *fp)
+perfmgr_db_print_by_guid(perfmgr_db_t * db, uint64_t nodeguid, FILE *fp,
+			 char *port, int err_only)
 {
 	cl_map_item_t *node;
 
@@ -732,7 +942,7 @@ perfmgr_db_print_by_guid(perfmgr_db_t * db, uint64_t nodeguid, FILE *fp)
 
 	node = cl_qmap_get(&db->pc_data, nodeguid);
 	if (node != cl_qmap_end(&db->pc_data))
-		dump_node_hr((db_node_t *)node, fp);
+		dump_node_hr((db_node_t *)node, fp, port, err_only);
 	else
 		fprintf(fp, "Node 0x%" PRIx64 " not found...\n", nodeguid);
 
@@ -767,15 +977,14 @@ perfmgr_db_fill_err_read(ib_port_counters_t * wire_read,
 			 perfmgr_db_err_reading_t * reading)
 {
 	reading->symbol_err_cnt = cl_ntoh16(wire_read->symbol_err_cnt);
-	reading->link_err_recover = cl_ntoh16(wire_read->link_err_recover);
+	reading->link_err_recover = wire_read->link_err_recover;
 	reading->link_downed = wire_read->link_downed;
-	reading->rcv_err = wire_read->rcv_err;
+	reading->rcv_err = cl_ntoh16(wire_read->rcv_err);
 	reading->rcv_rem_phys_err = cl_ntoh16(wire_read->rcv_rem_phys_err);
 	reading->rcv_switch_relay_err =
 	    cl_ntoh16(wire_read->rcv_switch_relay_err);
 	reading->xmit_discards = cl_ntoh16(wire_read->xmit_discards);
-	reading->xmit_constraint_err =
-	    cl_ntoh16(wire_read->xmit_constraint_err);
+	reading->xmit_constraint_err = wire_read->xmit_constraint_err;
 	reading->rcv_constraint_err = wire_read->rcv_constraint_err;
 	reading->link_integrity =
 	    PC_LINK_INT(wire_read->link_int_buffer_overrun);
