@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
- * Copyright (c) 2002-2009 Mellanox Technologies LTD. All rights reserved.
+ * Copyright (c) 2002-2012 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
  * Copyright (c) 2008 Xsigo Systems Inc.  All rights reserved.
  * Copyright (c) 2009 HNR Consulting.  All rights reserved.
@@ -48,6 +48,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <arpa/inet.h>
+#include <opensm/osm_file_ids.h>
+#define FILE_ID OSM_FILE_QOS_POLICY_C
 #include <opensm/osm_log.h>
 #include <opensm/osm_node.h>
 #include <opensm/osm_port.h>
@@ -311,7 +314,8 @@ boolean_t osm_qos_level_has_pkey(IN const osm_qos_level_t * p_qos_level,
 
 ib_net16_t osm_qos_level_get_shared_pkey(IN const osm_qos_level_t * p_qos_level,
 					 IN const osm_physp_t * p_src_physp,
-					 IN const osm_physp_t * p_dest_physp)
+					 IN const osm_physp_t * p_dest_physp,
+					 IN const boolean_t allow_both_pkeys)
 {
 	unsigned i;
 	uint16_t pkey_ho = 0;
@@ -329,7 +333,8 @@ ib_net16_t osm_qos_level_get_shared_pkey(IN const osm_qos_level_t * p_qos_level,
 		for (pkey_ho = p_qos_level->pkey_range_arr[i][0];
 		     pkey_ho <= p_qos_level->pkey_range_arr[i][1]; pkey_ho++) {
 			if (osm_physp_share_this_pkey
-			    (p_src_physp, p_dest_physp, cl_hton16(pkey_ho)))
+			    (p_src_physp, p_dest_physp, cl_hton16(pkey_ho),
+			     allow_both_pkeys))
 				return cl_hton16(pkey_ho);
 		}
 	}
@@ -577,6 +582,7 @@ static osm_qos_match_rule_t *__qos_policy_get_match_rule_by_params(
 
 	boolean_t matched_by_sguid = FALSE,
 		  matched_by_dguid = FALSE,
+		  matched_by_sordguid = FALSE,
 		  matched_by_class = FALSE,
 		  matched_by_sid = FALSE,
 		  matched_by_pkey = FALSE;
@@ -597,9 +603,11 @@ static osm_qos_match_rule_t *__qos_policy_get_match_rule_by_params(
 			continue;
 		}
 
-		/* If a match rule has Source groups, PR request source has to be in this list */
+		/* If a match rule has Source groups and no Destination groups,
+		 * PR request source has to be in this list */
 
-		if (cl_list_count(&p_qos_match_rule->source_group_list)) {
+		if (cl_list_count(&p_qos_match_rule->source_group_list)
+		    && !cl_list_count(&p_qos_match_rule->destination_group_list)) {
 			if (!__qos_policy_is_port_in_group_list(p_qos_policy,
 								p_src_physp,
 								&p_qos_match_rule->
@@ -611,9 +619,11 @@ static osm_qos_match_rule_t *__qos_policy_get_match_rule_by_params(
 			matched_by_sguid = TRUE;
 		}
 
-		/* If a match rule has Destination groups, PR request dest. has to be in this list */
+		/* If a match rule has Destination groups and no Source groups,
+		 * PR request dest. has to be in this list */
 
-		if (cl_list_count(&p_qos_match_rule->destination_group_list)) {
+		if (cl_list_count(&p_qos_match_rule->destination_group_list)
+		    && !cl_list_count(&p_qos_match_rule->source_group_list)) {
 			if (!__qos_policy_is_port_in_group_list(p_qos_policy,
 								p_dest_physp,
 								&p_qos_match_rule->
@@ -625,6 +635,26 @@ static osm_qos_match_rule_t *__qos_policy_get_match_rule_by_params(
 			matched_by_dguid = TRUE;
 		}
 
+		/* If a match rule has both Source and Destination groups,
+		 * PR request source or dest. must be in respective list
+		 */
+		if (cl_list_count(&p_qos_match_rule->source_group_list)
+		    && cl_list_count(&p_qos_match_rule->destination_group_list)) {
+			if (!__qos_policy_is_port_in_group_list(p_qos_policy,
+								p_src_physp,
+								&p_qos_match_rule->
+								source_group_list)
+			    && !__qos_policy_is_port_in_group_list(p_qos_policy,
+								   p_dest_physp,
+								   &p_qos_match_rule->
+								   destination_group_list))
+			{
+				list_iterator = cl_list_next(list_iterator);
+				continue;
+			}
+			matched_by_sordguid = TRUE;
+		}
+
 		/* If a match rule has QoS classes, PR request HAS
 		   to have a matching QoS class to match the rule */
 
@@ -692,11 +722,12 @@ static osm_qos_match_rule_t *__qos_policy_get_match_rule_by_params(
 
 	if (p_qos_match_rule)
 		OSM_LOG(p_log, OSM_LOG_DEBUG,
-			"request matched rule (%s) by:%s%s%s%s%s\n",
+			"request matched rule (%s) by:%s%s%s%s%s%s\n",
 			(p_qos_match_rule->use) ?
 				p_qos_match_rule->use : "no description",
 			(matched_by_sguid) ? " SGUID" : "",
 			(matched_by_dguid) ? " DGUID" : "",
+			(matched_by_sordguid) ? "SorDGUID" : "",
 			(matched_by_class) ? " QoS_Class" : "",
 			(matched_by_sid)   ? " ServiceID" : "",
 			(matched_by_pkey)  ? " PKey" : "");
@@ -769,9 +800,7 @@ static void __qos_policy_validate_pkey(
 			osm_qos_match_rule_t * p_qos_match_rule,
 			osm_prtn_t * p_prtn)
 {
-	uint8_t sl;
-	uint32_t flow;
-	uint8_t hop;
+	int i = 0;
 
 	if (!p_qos_policy || !p_qos_match_rule || !p_prtn)
 		return;
@@ -789,24 +818,30 @@ static void __qos_policy_validate_pkey(
 	p_prtn->sl = p_qos_match_rule->p_qos_level->sl;
 
 
-	/* If this partition is an IPoIB partition, there should
-	   be a matching MCast group. Fix this group's SL too */
-	if (!p_prtn->mgrp)
-		return;
+	/* If this partition has default MCast groups.
+	 * Fix those group's SL too */
+	for (i = 0; i < p_prtn->nmgrps; i++)
+	{
+		uint8_t sl;
+		uint32_t flow;
+		uint8_t hop;
 
-	CL_ASSERT((cl_ntoh16(p_prtn->mgrp->mcmember_rec.pkey) & 0x7fff) ==
-		  (cl_ntoh16(p_prtn->pkey) & 0x7fff));
+		CL_ASSERT((cl_ntoh16(p_prtn->mgrps[i]->mcmember_rec.pkey) & 0x7fff) ==
+			  (cl_ntoh16(p_prtn->pkey) & 0x7fff));
 
-	ib_member_get_sl_flow_hop(p_prtn->mgrp->mcmember_rec.sl_flow_hop,
-				  &sl, &flow, &hop);
-	if (sl != p_prtn->sl) {
-		OSM_LOG(&p_qos_policy->p_subn->p_osm->log, OSM_LOG_DEBUG,
-			"Updating MCGroup (MLID 0x%04x) SL to "
-			"match partition SL (%u)\n",
-			cl_hton16(p_prtn->mgrp->mcmember_rec.mlid),
-			p_prtn->sl);
-		p_prtn->mgrp->mcmember_rec.sl_flow_hop =
-			ib_member_set_sl_flow_hop(p_prtn->sl, flow, hop);
+		ib_member_get_sl_flow_hop(p_prtn->mgrps[i]->mcmember_rec.sl_flow_hop,
+					  &sl, &flow, &hop);
+		if (sl != p_prtn->sl) {
+			char gid_str[INET6_ADDRSTRLEN];
+			OSM_LOG(&p_qos_policy->p_subn->p_osm->log, OSM_LOG_DEBUG,
+				"Updating MCGroup (MGID %s) SL to "
+				"match partition SL (%u)\n",
+				inet_ntop(AF_INET6, p_prtn->mgrps[i]->mcmember_rec.mgid.raw,
+					  gid_str, sizeof gid_str),
+				p_prtn->sl);
+			p_prtn->mgrps[i]->mcmember_rec.sl_flow_hop =
+				ib_member_set_sl_flow_hop(p_prtn->sl, flow, hop);
+		}
 	}
 }
 
