@@ -52,6 +52,8 @@
 #include <complib/cl_passivelock.h>
 #include <complib/cl_debug.h>
 #include <complib/cl_qlist.h>
+#include <opensm/osm_file_ids.h>
+#define FILE_ID OSM_FILE_SA_INFORMINFO_C
 #include <vendor/osm_vendor_api.h>
 #include <opensm/osm_port.h>
 #include <opensm/osm_node.h>
@@ -89,12 +91,8 @@ static boolean_t validate_ports_access_rights(IN osm_sa_t * sa,
 	boolean_t valid = TRUE;
 	osm_physp_t *p_requester_physp;
 	osm_port_t *p_port;
-	osm_physp_t *p_physp;
 	ib_net64_t portguid;
-	ib_net16_t lid_range_begin;
-	ib_net16_t lid_range_end;
-	ib_net16_t lid;
-	const cl_ptr_vector_t *p_tbl;
+	uint16_t lid_range_begin, lid_range_end, lid;
 
 	OSM_LOG_ENTER(sa->p_log);
 
@@ -109,7 +107,6 @@ static boolean_t validate_ports_access_rights(IN osm_sa_t * sa,
 		    interface_id;
 
 		p_port = osm_get_port_by_guid(sa->p_subn, portguid);
-
 		if (p_port == NULL) {
 			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4301: "
 				"Invalid port guid: 0x%016" PRIx64 "\n",
@@ -118,19 +115,19 @@ static boolean_t validate_ports_access_rights(IN osm_sa_t * sa,
 			goto Exit;
 		}
 
-		/* get the destination InformInfo physical port */
-		p_physp = p_port->p_physp;
-
 		/* make sure that the requester and destination port can access
 		   each other according to the current partitioning. */
 		if (!osm_physp_share_pkey
-		    (sa->p_log, p_physp, p_requester_physp)) {
+		    (sa->p_log, p_port->p_physp, p_requester_physp,
+		     sa->p_subn->opt.allow_both_pkeys)) {
 			OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
 				"port and requester don't share pkey\n");
 			valid = FALSE;
 			goto Exit;
 		}
 	} else {
+		size_t lids_size;
+
 		/* gid is zero - check if LID range is defined */
 		lid_range_begin =
 		    cl_ntoh16(p_infr_rec->inform_record.inform_info.
@@ -143,33 +140,36 @@ static boolean_t validate_ports_access_rights(IN osm_sa_t * sa,
 		    cl_ntoh16(p_infr_rec->inform_record.inform_info.
 			      lid_range_end);
 
+		lids_size = cl_ptr_vector_get_size(&sa->p_subn->port_lid_tbl);
+
 		/* lid_range_end is set to zero if no range desired. In this
 		   case - just make it equal to the lid_range_begin. */
 		if (lid_range_end == 0)
 			lid_range_end = lid_range_begin;
+		else if (lid_range_end >= lids_size)
+			lid_range_end = lids_size - 1;
+
+		if (lid_range_begin >= lids_size) {
+			/* requested lids are out of range */
+			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4302: "
+				"Given LIDs (%u-%u) are out of range (%zu)\n",
+				lid_range_begin, lid_range_end, lids_size);
+			valid = FALSE;
+			goto Exit;
+		}
 
 		/* go over all defined lids within the range and make sure that the
 		   requester port can access them according to current partitioning. */
 		for (lid = lid_range_begin; lid <= lid_range_end; lid++) {
-			p_tbl = &sa->p_subn->port_lid_tbl;
-			if (cl_ptr_vector_get_size(p_tbl) > lid)
-				p_port = cl_ptr_vector_get(p_tbl, lid);
-			else {
-				/* lid requested is out of range */
-				OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4302: "
-					"Given LID (%u) is out of range:%u\n",
-					lid, cl_ptr_vector_get_size(p_tbl));
-				valid = FALSE;
-				goto Exit;
-			}
+			p_port = osm_get_port_by_lid_ho(sa->p_subn, lid);
 			if (p_port == NULL)
 				continue;
 
-			p_physp = p_port->p_physp;
 			/* make sure that the requester and destination port can access
 			   each other according to the current partitioning. */
 			if (!osm_physp_share_pkey
-			    (sa->p_log, p_physp, p_requester_physp)) {
+			    (sa->p_log, p_port->p_physp, p_requester_physp,
+			     sa->p_subn->opt.allow_both_pkeys)) {
 				OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
 					"port and requester don't share pkey\n");
 				valid = FALSE;
@@ -239,7 +239,6 @@ static void sa_inform_info_rec_by_comp_mask(IN osm_sa_t * sa,
 					    IN const osm_infr_t * p_infr,
 					    osm_iir_search_ctxt_t * p_ctxt)
 {
-	const ib_inform_info_record_t *p_rcvd_rec = NULL;
 	ib_net64_t comp_mask;
 	ib_net64_t portguid;
 	osm_port_t *p_subscriber_port;
@@ -249,7 +248,6 @@ static void sa_inform_info_rec_by_comp_mask(IN osm_sa_t * sa,
 
 	OSM_LOG_ENTER(sa->p_log);
 
-	p_rcvd_rec = p_ctxt->p_rcvd_rec;
 	comp_mask = p_ctxt->comp_mask;
 	p_req_physp = p_ctxt->p_req_physp;
 
@@ -279,7 +277,8 @@ static void sa_inform_info_rec_by_comp_mask(IN osm_sa_t * sa,
 	p_subscriber_physp = p_subscriber_port->p_physp;
 	/* make sure that the requester and subscriber port can access each
 	   other according to the current partitioning. */
-	if (!osm_physp_share_pkey(sa->p_log, p_req_physp, p_subscriber_physp)) {
+	if (!osm_physp_share_pkey(sa->p_log, p_req_physp, p_subscriber_physp,
+				  sa->p_subn->opt.allow_both_pkeys)) {
 		OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
 			"requester and subscriber ports don't share pkey\n");
 		goto Exit;
@@ -329,7 +328,7 @@ static void infr_rcv_process_get_method(osm_sa_t * sa, IN osm_madw_t * p_madw)
 	p_rcvd_rec =
 	    (ib_inform_info_record_t *) ib_sa_mad_get_payload_ptr(p_rcvd_mad);
 
-	/* update the requester physical port. */
+	/* update the requester physical port */
 	p_req_physp = osm_get_physp_by_mad_addr(sa->p_log, sa->p_subn,
 						osm_madw_get_mad_addr_ptr
 						(p_madw));
@@ -339,9 +338,13 @@ static void infr_rcv_process_get_method(osm_sa_t * sa, IN osm_madw_t * p_madw)
 		goto Exit;
 	}
 
-	if (osm_log_is_active(sa->p_log, OSM_LOG_DEBUG))
-		osm_dump_inform_info_record(sa->p_log, p_rcvd_rec,
-					    OSM_LOG_DEBUG);
+	if (OSM_LOG_IS_ACTIVE_V2(sa->p_log, OSM_LOG_DEBUG)) {
+		OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
+			"Requester port GUID 0x%" PRIx64 "\n",
+			cl_ntoh64(osm_physp_get_port_guid(p_req_physp)));
+		osm_dump_inform_info_record_v2(sa->p_log, p_rcvd_rec,
+					       FILE_ID, OSM_LOG_DEBUG);
+	}
 
 	cl_qlist_init(&rec_list);
 
@@ -404,9 +407,9 @@ static void infr_rcv_process_set_method(osm_sa_t * sa, IN osm_madw_t * p_madw)
 	    (ib_inform_info_t *) ib_sa_mad_get_payload_ptr(p_sa_mad);
 
 #if 0
-	if (osm_log_is_active(sa->p_log, OSM_LOG_DEBUG))
-		osm_dump_inform_info(sa->p_log, p_recvd_inform_info,
-				     OSM_LOG_DEBUG);
+	if (OSM_LOG_IS_ACTIVE_V2(sa->p_log, OSM_LOG_DEBUG))
+		osm_dump_inform_info_v2(sa->p_log, p_recvd_inform_info,
+				        FILE_ID, OSM_LOG_DEBUG);
 #endif
 
 	/* Grab the lock */
@@ -429,7 +432,7 @@ static void infr_rcv_process_set_method(osm_sa_t * sa, IN osm_madw_t * p_madw)
 	if (res != IB_SUCCESS) {
 		cl_plock_release(sa->p_lock);
 
-		OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4308 "
+		OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4308: "
 			"Subscribe Request from unknown LID: %u\n",
 			cl_ntoh16(p_madw->mad_addr.dest_lid));
 		osm_sa_send_error(sa, p_madw, IB_SA_MAD_STATUS_REQ_INVALID);
@@ -443,7 +446,7 @@ static void infr_rcv_process_set_method(osm_sa_t * sa, IN osm_madw_t * p_madw)
 	if (p_recvd_inform_info->subscribe > 1) {
 		cl_plock_release(sa->p_lock);
 
-		OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4308 "
+		OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 430A: "
 			"Invalid subscribe: %d\n",
 			p_recvd_inform_info->subscribe);
 		osm_sa_send_error(sa, p_madw, IB_SA_MAD_STATUS_REQ_INVALID);
