@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
- * Copyright (c) 2002-2009 Mellanox Technologies LTD. All rights reserved.
+ * Copyright (c) 2002-2011 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
  * Copyright (c) 2008 Xsigo Systems Inc.  All rights reserved.
  *
@@ -53,6 +53,8 @@
 #include <complib/cl_passivelock.h>
 #include <complib/cl_debug.h>
 #include <complib/cl_qlist.h>
+#include <opensm/osm_file_ids.h>
+#define FILE_ID OSM_FILE_SA_MCMEMBER_RECORD_C
 #include <vendor/osm_vendor_api.h>
 #include <opensm/osm_madw.h>
 #include <opensm/osm_log.h>
@@ -285,7 +287,7 @@ static boolean_t validate_more_comp_fields(osm_log_t * p_log,
 		rate_mgrp = (uint8_t) (p_mgrp->mcmember_rec.rate & 0x3F);
 		switch (rate_sel) {
 		case 0:	/* Greater than RATE specified */
-			if (rate_mgrp <= rate_required) {
+			if (ib_path_compare_rates(rate_mgrp, rate_required) <= 0) {
 				OSM_LOG(p_log, OSM_LOG_VERBOSE,
 					"Requested mcast group has RATE %x, "
 					"which is not greater than %x\n",
@@ -294,7 +296,7 @@ static boolean_t validate_more_comp_fields(osm_log_t * p_log,
 			}
 			break;
 		case 1:	/* Less than RATE specified */
-			if (rate_mgrp >= rate_required) {
+			if (ib_path_compare_rates(rate_mgrp, rate_required) >= 0) {
 				OSM_LOG(p_log, OSM_LOG_VERBOSE,
 					"Requested mcast group has RATE %x, "
 					"which is not less than %x\n",
@@ -303,7 +305,7 @@ static boolean_t validate_more_comp_fields(osm_log_t * p_log,
 			}
 			break;
 		case 2:	/* Exactly RATE specified */
-			if (rate_mgrp != rate_required) {
+			if (ib_path_compare_rates(rate_mgrp, rate_required)) {
 				OSM_LOG(p_log, OSM_LOG_VERBOSE,
 					"Requested mcast group has RATE %x, "
 					"which is not equal to %x\n",
@@ -327,6 +329,7 @@ static boolean_t validate_port_caps(osm_log_t * p_log,
 				    const osm_mgrp_t * p_mgrp,
 				    const osm_physp_t * p_physp)
 {
+	const ib_port_info_t *p_pi;
 	uint8_t mtu_required;
 	uint8_t mtu_mgrp;
 	uint8_t rate_required;
@@ -341,9 +344,11 @@ static boolean_t validate_port_caps(osm_log_t * p_log,
 		return FALSE;
 	}
 
-	rate_required = ib_port_info_compute_rate(&p_physp->port_info);
+	p_pi = &p_physp->port_info;
+	rate_required = ib_port_info_compute_rate(p_pi,
+						  p_pi->capability_mask & IB_PORT_CAP_HAS_EXT_SPEEDS);
 	rate_mgrp = (uint8_t) (p_mgrp->mcmember_rec.rate & 0x3F);
-	if (rate_required < rate_mgrp) {
+	if (ib_path_compare_rates(rate_required, rate_mgrp) < 0) {
 		OSM_LOG(p_log, OSM_LOG_VERBOSE,
 			"Port's RATE %x is less than %x\n",
 			rate_required, rate_mgrp);
@@ -367,7 +372,7 @@ static boolean_t validate_port_caps(osm_log_t * p_log,
 static boolean_t validate_modify(IN osm_sa_t * sa, IN osm_mgrp_t * p_mgrp,
 				 IN osm_mad_addr_t * p_mad_addr,
 				 IN ib_member_rec_t * p_recvd_mcmember_rec,
-				 OUT osm_mcm_port_t ** pp_mcm_port)
+				 OUT osm_mcm_alias_guid_t ** pp_mcm_alias_guid)
 {
 	ib_net64_t portguid;
 	ib_gid_t request_gid;
@@ -376,10 +381,10 @@ static boolean_t validate_modify(IN osm_sa_t * sa, IN osm_mgrp_t * p_mgrp,
 
 	portguid = p_recvd_mcmember_rec->port_gid.unicast.interface_id;
 
-	*pp_mcm_port = osm_mgrp_get_mcm_port(p_mgrp, portguid);
+	*pp_mcm_alias_guid = osm_mgrp_get_mcm_alias_guid(p_mgrp, portguid);
 
 	/* o15-0.2.1: If this is a new port being added - nothing to check */
-	if (!*pp_mcm_port) {
+	if (!*pp_mcm_alias_guid) {
 		OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
 			"This is a new port in the MC group\n");
 		return TRUE;
@@ -387,7 +392,7 @@ static boolean_t validate_modify(IN osm_sa_t * sa, IN osm_mgrp_t * p_mgrp,
 
 	/* We validate the request according the the proxy_join.
 	   Check if the proxy_join is set or not */
-	if ((*pp_mcm_port)->proxy_join == FALSE) {
+	if ((*pp_mcm_alias_guid)->proxy_join == FALSE) {
 		/* The proxy_join is not set. Modifying can by done only
 		   if the requester GID == PortGID */
 		res = osm_get_gid_by_mad_addr(sa->p_log, sa->p_subn, p_mad_addr,
@@ -398,15 +403,15 @@ static boolean_t validate_modify(IN osm_sa_t * sa, IN osm_mgrp_t * p_mgrp,
 			return FALSE;
 		}
 
-		if (memcmp(&(*pp_mcm_port)->port_gid, &request_gid,
-			   sizeof(ib_gid_t))) {
+		if ((*pp_mcm_alias_guid)->p_base_mcm_port->port->guid !=
+		    request_gid.unicast.interface_id ||
+		    (*pp_mcm_alias_guid)->port_gid.unicast.prefix !=
+		    request_gid.unicast.prefix) {
 			OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
 				"No ProxyJoin but different ports: stored:"
 				"0x%016" PRIx64 " request:0x%016" PRIx64 "\n",
-				cl_ntoh64((*pp_mcm_port)->port_gid.unicast.
-					  interface_id),
-				cl_ntoh64(p_mad_addr->addr_type.gsi.grh_info.
-					  src_gid.unicast.interface_id));
+				cl_ntoh64((*pp_mcm_alias_guid)->p_base_mcm_port->port->guid),
+				cl_ntoh64(request_gid.unicast.interface_id));
 			return FALSE;
 		}
 	} else {
@@ -422,11 +427,9 @@ static boolean_t validate_modify(IN osm_sa_t * sa, IN osm_mgrp_t * p_mgrp,
 					p_request_physp)) {
 			/* the request port is not part of the partition for this mgrp */
 			OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
-				"ProxyJoin but port not in partition. stored:"
-				"0x%016" PRIx64 " request:0x%016" PRIx64 "\n",
-				cl_ntoh64((*pp_mcm_port)->port->guid),
-				cl_ntoh64(p_mad_addr->addr_type.gsi.grh_info.
-					  src_gid.unicast.interface_id));
+				"Requesting port 0x%016" PRIx64 " has no P_Key 0x%04x\n",
+				cl_ntoh64(p_request_physp->port_guid),
+				cl_ntoh16(p_mgrp->mcmember_rec.pkey));
 			return FALSE;
 		}
 	}
@@ -456,16 +459,16 @@ static boolean_t validate_modify(IN osm_sa_t * sa, IN osm_mgrp_t * p_mgrp,
 static boolean_t validate_delete(IN osm_sa_t * sa, IN osm_mgrp_t * p_mgrp,
 				 IN osm_mad_addr_t * p_mad_addr,
 				 IN ib_member_rec_t * p_recvd_mcmember_rec,
-				 OUT osm_mcm_port_t ** pp_mcm_port)
+				 OUT osm_mcm_alias_guid_t ** pp_mcm_alias_guid)
 {
 	ib_net64_t portguid;
 
 	portguid = p_recvd_mcmember_rec->port_gid.unicast.interface_id;
 
-	*pp_mcm_port = osm_mgrp_get_mcm_port(p_mgrp, portguid);
+	*pp_mcm_alias_guid = osm_mgrp_get_mcm_alias_guid(p_mgrp, portguid);
 
 	/* 1 */
-	if (!*pp_mcm_port) {
+	if (!*pp_mcm_alias_guid) {
 		OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
 			"Failed to find the port in the MC group\n");
 		return FALSE;
@@ -473,7 +476,7 @@ static boolean_t validate_delete(IN osm_sa_t * sa, IN osm_mgrp_t * p_mgrp,
 
 	/* 2 */
 	if (!(p_recvd_mcmember_rec->scope_state & 0x0F &
-	      (*pp_mcm_port)->scope_state)) {
+	      (*pp_mcm_alias_guid)->scope_state)) {
 		OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
 			"Could not find any matching bits in the stored "
 			"and requested JoinStates\n");
@@ -482,20 +485,20 @@ static boolean_t validate_delete(IN osm_sa_t * sa, IN osm_mgrp_t * p_mgrp,
 
 	/* 3 */
 	if (((p_recvd_mcmember_rec->scope_state & 0x0F) |
-	     (0x0F & (*pp_mcm_port)->scope_state)) !=
-	    (0x0F & (*pp_mcm_port)->scope_state)) {
+	     (0x0F & (*pp_mcm_alias_guid)->scope_state)) !=
+	    (0x0F & (*pp_mcm_alias_guid)->scope_state)) {
 		OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
 			"Some bits in the request JoinState (0x%X) are not "
 			"set in the stored port (0x%X)\n",
 			(p_recvd_mcmember_rec->scope_state & 0x0F),
-			(0x0F & (*pp_mcm_port)->scope_state));
+			(0x0F & (*pp_mcm_alias_guid)->scope_state));
 		return FALSE;
 	}
 
 	/* 4 */
 	/* Validate according the the proxy_join (o15-0.1.2) */
 	if (validate_modify(sa, p_mgrp, p_mad_addr, p_recvd_mcmember_rec,
-			    pp_mcm_port) == FALSE) {
+			    pp_mcm_alias_guid) == FALSE) {
 		OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
 			"proxy_join validation failure\n");
 		return FALSE;
@@ -615,7 +618,7 @@ static ib_api_status_t validate_requested_mgid(IN osm_sa_t * sa,
 
 	/*
 	 * For SA assigned MGIDs (signature 0xA01B):
-	 * There is no real way to make sure the Unique MGID Prefix is really unique.
+	 * There is no real way to make sure the GID Prefix is really unique.
 	 * If we could enforce using the Subnet Prefix for that purpose it would
 	 * have been nice. But the spec does not require it.
 	 */
@@ -638,6 +641,7 @@ static boolean_t mgrp_request_is_realizable(IN osm_sa_t * sa,
 	uint8_t mtu_required, mtu, port_mtu;
 	uint8_t rate_sel = 2;	/* exactly */
 	uint8_t rate_required, rate, port_rate;
+	const ib_port_info_t *p_pi;
 	osm_log_t *p_log = sa->p_log;
 
 	OSM_LOG_ENTER(sa->p_log);
@@ -655,7 +659,8 @@ static boolean_t mgrp_request_is_realizable(IN osm_sa_t * sa,
 	 * masked in.
 	 */
 
-	port_mtu = p_physp ? ib_port_info_get_mtu_cap(&p_physp->port_info) : 0;
+	p_pi = &p_physp->port_info;
+	port_mtu = p_physp ? ib_port_info_get_mtu_cap(p_pi) : 0;
 	if (!(comp_mask & IB_MCR_COMPMASK_MTU) ||
 	    !(comp_mask & IB_MCR_COMPMASK_MTU_SEL) ||
 	    (mtu_sel = (p_mcm_rec->mtu >> 6)) == 3)
@@ -692,7 +697,7 @@ static boolean_t mgrp_request_is_realizable(IN osm_sa_t * sa,
 		default:
 			break;
 		}
-		/* make sure it still be in the range */
+		/* make sure it still is in the range */
 		if (mtu < IB_MIN_MTU || mtu > IB_MAX_MTU) {
 			OSM_LOG(p_log, OSM_LOG_VERBOSE,
 				"Calculated MTU %x is out of range\n", mtu);
@@ -702,7 +707,8 @@ static boolean_t mgrp_request_is_realizable(IN osm_sa_t * sa,
 	p_mcm_rec->mtu = (mtu_sel << 6) | mtu;
 
 	port_rate =
-	    p_physp ? ib_port_info_compute_rate(&p_physp->port_info) : 0;
+	    p_physp ? ib_port_info_compute_rate(p_pi,
+						p_pi->capability_mask & IB_PORT_CAP_HAS_EXT_SPEEDS) : 0;
 	if (!(comp_mask & IB_MCR_COMPMASK_RATE)
 	    || !(comp_mask & IB_MCR_COMPMASK_RATE_SEL)
 	    || (rate_sel = (p_mcm_rec->rate >> 6)) == 3)
@@ -712,7 +718,7 @@ static boolean_t mgrp_request_is_realizable(IN osm_sa_t * sa,
 		rate = rate_required;
 		switch (rate_sel) {
 		case 0:	/* Greater than RATE specified */
-			if (port_rate && rate_required >= port_rate) {
+			if (ib_path_compare_rates(rate_required, port_rate) >= 0) {
 				OSM_LOG(p_log, OSM_LOG_VERBOSE,
 					"Requested RATE %x >= the port\'s rate:%x\n",
 					rate_required, port_rate);
@@ -721,19 +727,20 @@ static boolean_t mgrp_request_is_realizable(IN osm_sa_t * sa,
 			/* we provide the largest RATE possible if we can */
 			if (port_rate)
 				rate = port_rate;
-			else if (rate_required < sa->p_subn->min_ca_rate)
+			else if (ib_path_compare_rates(rate_required,
+						       sa->p_subn->min_ca_rate) < 0)
 				rate = sa->p_subn->min_ca_rate;
 			else
-				rate++;
+				rate = ib_path_rate_get_next(rate);
 			break;
 		case 1:	/* Less than RATE specified */
 			/* use the smaller of the two:
 			   a. one lower then the required
 			   b. the rate of the requesting port (if exists) */
-			if (port_rate && rate_required > port_rate)
+			if (ib_path_compare_rates(rate_required, port_rate) > 0)
 				rate = port_rate;
 			else
-				rate--;
+				rate = ib_path_rate_get_prev(rate);
 			break;
 		case 2:	/* Exactly RATE specified */
 		default:
@@ -772,7 +779,6 @@ static unsigned build_new_mgid(osm_sa_t * sa, ib_net64_t comp_mask,
 	mgid->raw[2] = 0xa0;
 	mgid->raw[3] = 0x1b;
 
-	/* HACK: use the SA port gid to make it globally unique */
 	memcpy(&mgid->raw[4], &sa->p_subn->opt.subnet_prefix, sizeof(uint64_t));
 
 	for (i = 0; i < 1000; i++) {
@@ -871,21 +877,20 @@ Exit:
 /**********************************************************************
  Call this function to find or create a new mgrp.
 **********************************************************************/
-ib_api_status_t osm_mcmr_rcv_find_or_create_new_mgrp(IN osm_sa_t * sa,
-						     IN ib_net64_t comp_mask,
-						     IN ib_member_rec_t *
-						     p_recvd_mcmember_rec,
-						     OUT osm_mgrp_t ** pp_mgrp)
+osm_mgrp_t *osm_mcmr_rcv_find_or_create_new_mgrp(IN osm_sa_t * sa,
+						 IN ib_net64_t comp_mask,
+						 IN ib_member_rec_t *
+						 p_recvd_mcmember_rec)
 {
 	osm_mgrp_t *mgrp;
 
 	if ((mgrp = osm_get_mgrp_by_mgid(sa->p_subn,
-					 &p_recvd_mcmember_rec->mgid))) {
-		*pp_mgrp = mgrp;
-		return IB_SUCCESS;
-	}
-	return mcmr_rcv_create_new_mgrp(sa, comp_mask, p_recvd_mcmember_rec,
-					NULL, pp_mgrp);
+					 &p_recvd_mcmember_rec->mgid)))
+		return mgrp;
+	if (mcmr_rcv_create_new_mgrp(sa, comp_mask, p_recvd_mcmember_rec, NULL,
+				     &mgrp) == IB_SUCCESS)
+		return mgrp;
+	return NULL;
 }
 
 /*********************************************************************
@@ -897,8 +902,7 @@ static void mcmr_rcv_leave_mgrp(IN osm_sa_t * sa, IN osm_madw_t * p_madw)
 	ib_sa_mad_t *p_sa_mad;
 	ib_member_rec_t *p_recvd_mcmember_rec;
 	ib_member_rec_t mcmember_rec;
-	ib_net64_t portguid;
-	osm_mcm_port_t *p_mcm_port;
+	osm_mcm_alias_guid_t *p_mcm_alias_guid;
 
 	OSM_LOG_ENTER(sa->p_log);
 
@@ -908,9 +912,21 @@ static void mcmr_rcv_leave_mgrp(IN osm_sa_t * sa, IN osm_madw_t * p_madw)
 
 	mcmember_rec = *p_recvd_mcmember_rec;
 
-	if (osm_log_is_active(sa->p_log, OSM_LOG_DEBUG)) {
+	if (OSM_LOG_IS_ACTIVE_V2(sa->p_log, OSM_LOG_DEBUG)) {
+		osm_physp_t *p_req_physp;
+
+		p_req_physp = osm_get_physp_by_mad_addr(sa->p_log, sa->p_subn,
+							osm_madw_get_mad_addr_ptr(p_madw));
+		if (p_req_physp == NULL) {
+			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 1B02: "
+				"Cannot find requester physical port\n");
+		} else {
+			OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
+				"Requester port GUID 0x%" PRIx64 "\n",
+				cl_ntoh64(osm_physp_get_port_guid(p_req_physp)));
+		}
 		OSM_LOG(sa->p_log, OSM_LOG_DEBUG, "Dump of record\n");
-		osm_dump_mc_record(sa->p_log, &mcmember_rec, OSM_LOG_DEBUG);
+		osm_dump_mc_record_v2(sa->p_log, &mcmember_rec, FILE_ID, OSM_LOG_DEBUG);
 	}
 
 	CL_PLOCK_EXCL_ACQUIRE(sa->p_lock);
@@ -926,11 +942,9 @@ static void mcmr_rcv_leave_mgrp(IN osm_sa_t * sa, IN osm_madw_t * p_madw)
 		goto Exit;
 	}
 
-	portguid = p_recvd_mcmember_rec->port_gid.unicast.interface_id;
-
 	/* check validity of the delete request o15-0.1.14 */
 	if (!validate_delete(sa, p_mgrp, osm_madw_get_mad_addr_ptr(p_madw),
-			     p_recvd_mcmember_rec, &p_mcm_port)) {
+			     p_recvd_mcmember_rec, &p_mcm_alias_guid)) {
 		char gid_str[INET6_ADDRSTRLEN];
 		char gid_str2[INET6_ADDRSTRLEN];
 		CL_PLOCK_RELEASE(sa->p_lock);
@@ -946,7 +960,7 @@ static void mcmr_rcv_leave_mgrp(IN osm_sa_t * sa, IN osm_madw_t * p_madw)
 	}
 
 	/* remove port and/or update join state */
-	osm_mgrp_remove_port(sa->p_subn, sa->p_log, p_mgrp, p_mcm_port,
+	osm_mgrp_remove_port(sa->p_subn, sa->p_log, p_mgrp, p_mcm_alias_guid,
 			     &mcmember_rec);
 	CL_PLOCK_RELEASE(sa->p_lock);
 
@@ -967,6 +981,7 @@ static void mcmr_rcv_join_mgrp(IN osm_sa_t * sa, IN osm_madw_t * p_madw)
 	ib_member_rec_t *p_recvd_mcmember_rec;
 	ib_member_rec_t mcmember_rec;
 	osm_mcm_port_t *p_mcmr_port;
+	osm_mcm_alias_guid_t *p_mcm_alias_guid;
 	ib_net64_t portguid;
 	osm_port_t *p_port;
 	osm_physp_t *p_physp;
@@ -984,15 +999,27 @@ static void mcmr_rcv_join_mgrp(IN osm_sa_t * sa, IN osm_madw_t * p_madw)
 
 	mcmember_rec = *p_recvd_mcmember_rec;
 
-	if (osm_log_is_active(sa->p_log, OSM_LOG_DEBUG)) {
+	if (OSM_LOG_IS_ACTIVE_V2(sa->p_log, OSM_LOG_DEBUG)) {
+		osm_physp_t *p_req_physp;
+
+		p_req_physp = osm_get_physp_by_mad_addr(sa->p_log, sa->p_subn,
+							osm_madw_get_mad_addr_ptr(p_madw));
+		if (p_req_physp == NULL) {
+			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 1B03: "
+				"Cannot find requester physical port\n");
+		} else {
+			OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
+				"Requester port GUID 0x%" PRIx64 "\n",
+				cl_ntoh64(osm_physp_get_port_guid(p_req_physp)));
+		}
 		OSM_LOG(sa->p_log, OSM_LOG_DEBUG, "Dump of incoming record\n");
-		osm_dump_mc_record(sa->p_log, &mcmember_rec, OSM_LOG_DEBUG);
+		osm_dump_mc_record_v2(sa->p_log, &mcmember_rec, FILE_ID, OSM_LOG_DEBUG);
 	}
 
 	CL_PLOCK_EXCL_ACQUIRE(sa->p_lock);
 
 	/* make sure the requested port guid is known to the SM */
-	p_port = osm_get_port_by_guid(sa->p_subn, portguid);
+	p_port = osm_get_port_by_alias_guid(sa->p_subn, portguid);
 	if (!p_port) {
 		CL_PLOCK_RELEASE(sa->p_lock);
 		OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
@@ -1015,7 +1042,8 @@ static void mcmr_rcv_join_mgrp(IN osm_sa_t * sa, IN osm_madw_t * p_madw)
 
 	proxy = (p_physp != p_request_physp);
 
-	if (proxy && !osm_physp_share_pkey(sa->p_log, p_physp, p_request_physp)) {
+	if (proxy && !osm_physp_share_pkey(sa->p_log, p_physp, p_request_physp,
+					   sa->p_subn->opt.allow_both_pkeys)) {
 		CL_PLOCK_RELEASE(sa->p_lock);
 		OSM_LOG(sa->p_log, OSM_LOG_VERBOSE,
 			"Port and requester don't share pkey\n");
@@ -1023,6 +1051,15 @@ static void mcmr_rcv_join_mgrp(IN osm_sa_t * sa, IN osm_madw_t * p_madw)
 		goto Exit;
 	}
 
+	if ((p_sa_mad->comp_mask & IB_MCR_COMPMASK_PKEY) &&
+	    ib_pkey_is_invalid(p_recvd_mcmember_rec->pkey)) {
+		CL_PLOCK_RELEASE(sa->p_lock);
+		OSM_LOG(sa->p_log, OSM_LOG_VERBOSE,
+			"Invalid PKey supplied in request\n");
+		osm_sa_send_error(sa, p_madw, IB_SA_MAD_STATUS_REQ_INVALID);
+		goto Exit;
+	}
+
 	ib_member_get_scope_state(p_recvd_mcmember_rec->scope_state, NULL,
 				  &join_state);
 
@@ -1111,13 +1148,16 @@ static void mcmr_rcv_join_mgrp(IN osm_sa_t * sa, IN osm_madw_t * p_madw)
 				       p_sa_mad->comp_mask)
 	    || !validate_port_caps(sa->p_log, p_mgrp, p_physp)
 	    || !(join_state != 0)) {
+		char gid_str[INET6_ADDRSTRLEN];
 		/* since we might have created the new group we need to cleanup */
 		osm_mgrp_cleanup(sa->p_subn, p_mgrp);
 		CL_PLOCK_RELEASE(sa->p_lock);
 		OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 1B12: "
 			"validate_more_comp_fields, validate_port_caps, "
-			"or JoinState = 0 failed from port 0x%016" PRIx64
+			"or JoinState = 0 failed for MGID: %s port 0x%016" PRIx64
 			" (%s), " "sending IB_SA_MAD_STATUS_REQ_INVALID\n",
+			   inet_ntop(AF_INET6, p_mgrp->mcmember_rec.mgid.raw,
+				     gid_str, sizeof gid_str),
 			cl_ntoh64(portguid), p_port->p_node->print_desc);
 		osm_sa_send_error(sa, p_madw, IB_SA_MAD_STATUS_REQ_INVALID);
 		goto Exit;
@@ -1129,7 +1169,7 @@ static void mcmr_rcv_join_mgrp(IN osm_sa_t * sa, IN osm_madw_t * p_madw)
 	 */
 	if (!is_new_group &&
 	    !validate_modify(sa, p_mgrp, osm_madw_get_mad_addr_ptr(p_madw),
-			     p_recvd_mcmember_rec, &p_mcmr_port)) {
+			     p_recvd_mcmember_rec, &p_mcm_alias_guid)) {
 		CL_PLOCK_RELEASE(sa->p_lock);
 		OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 1B13: "
 			"validate_modify failed from port 0x%016" PRIx64
@@ -1158,8 +1198,8 @@ static void mcmr_rcv_join_mgrp(IN osm_sa_t * sa, IN osm_madw_t * p_madw)
 	/* Release the lock as we don't need it. */
 	CL_PLOCK_RELEASE(sa->p_lock);
 
-	if (osm_log_is_active(sa->p_log, OSM_LOG_DEBUG))
-		osm_dump_mc_record(sa->p_log, &mcmember_rec, OSM_LOG_DEBUG);
+	if (OSM_LOG_IS_ACTIVE_V2(sa->p_log, OSM_LOG_DEBUG))
+		osm_dump_mc_record_v2(sa->p_log, &mcmember_rec, FILE_ID, OSM_LOG_DEBUG);
 
 	mcmr_rcv_respond(sa, p_madw, &mcmember_rec);
 
@@ -1209,7 +1249,7 @@ static void mcmr_by_comp_mask(osm_sa_t * sa, const ib_member_rec_t * p_rcvd_rec,
 {
 	/* since we might change scope_state */
 	ib_member_rec_t match_rec;
-	osm_mcm_port_t *p_mcm_port;
+	osm_mcm_alias_guid_t *p_mcm_alias_guid;
 	ib_net64_t portguid = p_rcvd_rec->port_gid.unicast.interface_id;
 	/* will be used for group or port info */
 	uint8_t scope_state;
@@ -1289,12 +1329,13 @@ static void mcmr_by_comp_mask(osm_sa_t * sa, const ib_member_rec_t * p_rcvd_rec,
 	/* so did we get the PortGUID mask */
 	if (IB_MCR_COMPMASK_PORT_GID & comp_mask) {
 		/* try to find this port */
-		p_mcm_port = osm_mgrp_get_mcm_port(p_mgrp, portguid);
-		if (!p_mcm_port) /* port not in group */
+		p_mcm_alias_guid = osm_mgrp_get_mcm_alias_guid(p_mgrp, portguid);
+		if (!p_mcm_alias_guid) /* port not in group */
 			goto Exit;
-		scope_state = p_mcm_port->scope_state;
-		memcpy(&port_gid, &(p_mcm_port->port_gid), sizeof(ib_gid_t));
-		proxy_join = p_mcm_port->proxy_join;
+		scope_state = p_mcm_alias_guid->scope_state;
+		memcpy(&port_gid, &(p_mcm_alias_guid->port_gid),
+		       sizeof(ib_gid_t));
+		proxy_join = p_mcm_alias_guid->proxy_join;
 	} else /* point to the group information */
 		scope_state = p_mgrp->mcmember_rec.scope_state;
 
@@ -1311,17 +1352,18 @@ static void mcmr_by_comp_mask(osm_sa_t * sa, const ib_member_rec_t * p_rcvd_rec,
 			"Trusted req is TRUE and no specific port defined\n");
 
 		/* return all the ports that match in this MC group */
-		p_item = cl_qmap_head(&(p_mgrp->mcm_port_tbl));
-		while (p_item != cl_qmap_end(&(p_mgrp->mcm_port_tbl))) {
-			p_mcm_port = (osm_mcm_port_t *) p_item;
+		p_item = cl_qmap_head(&(p_mgrp->mcm_alias_port_tbl));
+		while (p_item != cl_qmap_end(&(p_mgrp->mcm_alias_port_tbl))) {
+			p_mcm_alias_guid = (osm_mcm_alias_guid_t *) p_item;
 
 			if ((scope_state_mask & p_rcvd_rec->scope_state) ==
-			    (scope_state_mask & p_mcm_port->scope_state)) {
+			    (scope_state_mask & p_mcm_alias_guid->scope_state)) {
 				/* add to the list */
 				match_rec = p_mgrp->mcmember_rec;
-				match_rec.scope_state = p_mcm_port->scope_state;
+				match_rec.scope_state = p_mcm_alias_guid->scope_state;
 				memcpy(&match_rec.port_gid,
-				       &p_mcm_port->port_gid, sizeof(ib_gid_t));
+				       &p_mcm_alias_guid->port_gid,
+				       sizeof(ib_gid_t));
 				OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
 					"Record of port_gid: %s"
 					" in multicast_lid: 0x%X is returned\n",
@@ -1331,7 +1373,7 @@ static void mcmr_by_comp_mask(osm_sa_t * sa, const ib_member_rec_t * p_rcvd_rec,
 					cl_ntoh16(p_mgrp->mlid));
 
 				match_rec.proxy_join =
-				    (uint8_t) (p_mcm_port->proxy_join);
+				    (uint8_t) (p_mcm_alias_guid->proxy_join);
 
 				mcmr_rcv_new_mcmr(sa, &match_rec, list);
 			}
@@ -1380,7 +1422,7 @@ static void mcmr_query_mgrp(IN osm_sa_t * sa, IN osm_madw_t * p_madw)
 	 */
 	trusted_req = (p_rcvd_mad->sm_key != 0);
 
-	/* update the requester physical port. */
+	/* update the requester physical port */
 	p_req_physp = osm_get_physp_by_mad_addr(sa->p_log, sa->p_subn,
 						osm_madw_get_mad_addr_ptr
 						(p_madw));
@@ -1390,6 +1432,14 @@ static void mcmr_query_mgrp(IN osm_sa_t * sa, IN osm_madw_t * p_madw)
 		goto Exit;
 	}
 
+	if (OSM_LOG_IS_ACTIVE_V2(sa->p_log, OSM_LOG_DEBUG)) {
+		OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
+			"Requester port GUID 0x%" PRIx64 "\n",
+			cl_ntoh64(osm_physp_get_port_guid(p_req_physp)));
+		OSM_LOG(sa->p_log, OSM_LOG_DEBUG, "Dump of record\n");
+		osm_dump_mc_record(sa->p_log, p_rcvd_rec, OSM_LOG_DEBUG);
+	}
+
 	cl_qlist_init(&rec_list);
 
 	CL_PLOCK_ACQUIRE(sa->p_lock);
@@ -1429,6 +1479,34 @@ Exit:
 	OSM_LOG_EXIT(sa->p_log);
 }
 
+static uint8_t rate_is_valid(IN const ib_sa_mad_t *p_sa_mad,
+			     IN const ib_member_rec_t *p_recvd_mcmember_rec)
+{
+	uint8_t rate;
+
+	/* Validate rate if supplied */
+	if ((p_sa_mad->comp_mask & IB_MCR_COMPMASK_RATE_SEL) &&
+	    (p_sa_mad->comp_mask & IB_MCR_COMPMASK_RATE)) {
+		rate = (uint8_t) (p_recvd_mcmember_rec->rate & 0x3F);
+		return ib_rate_is_valid(rate);
+	}
+	return 1;
+}
+
+static int mtu_is_valid(IN const ib_sa_mad_t *p_sa_mad,
+			IN const ib_member_rec_t *p_recvd_mcmember_rec)
+{
+	uint8_t mtu;
+
+	/* Validate MTU if supplied */
+	if ((p_sa_mad->comp_mask & IB_MCR_COMPMASK_MTU_SEL) &&
+	    (p_sa_mad->comp_mask & IB_MCR_COMPMASK_MTU)) {
+		mtu = (uint8_t) (p_recvd_mcmember_rec->mtu & 0x3F);
+		return ib_mtu_is_valid(mtu);
+	}
+	return 1;
+}
+
 void osm_mcmr_rcv_process(IN void *context, IN void *data)
 {
 	osm_sa_t *sa = context;
@@ -1465,6 +1543,12 @@ void osm_mcmr_rcv_process(IN void *context, IN void *data)
 				inet_ntop(AF_INET6,
 					  p_recvd_mcmember_rec->port_gid.raw,
 					  gid_str2, sizeof gid_str2));
+			osm_sa_send_error(sa, p_madw,
+					  IB_SA_MAD_STATUS_INSUF_COMPS);
+			goto Exit;
+		}
+		if (!rate_is_valid(p_sa_mad, p_recvd_mcmember_rec) ||
+		    !mtu_is_valid(p_sa_mad, p_recvd_mcmember_rec)) {
 			osm_sa_send_error(sa, p_madw,
 					  IB_SA_MAD_STATUS_REQ_INVALID);
 			goto Exit;
@@ -1482,6 +1566,12 @@ void osm_mcmr_rcv_process(IN void *context, IN void *data)
 				"expected comp mask = 0x%016" PRIx64 "\n",
 				cl_ntoh64(p_sa_mad->comp_mask),
 				CL_NTOH64(JOIN_MC_COMP_MASK));
+			osm_sa_send_error(sa, p_madw,
+					  IB_SA_MAD_STATUS_INSUF_COMPS);
+			goto Exit;
+		}
+		if (!rate_is_valid(p_sa_mad, p_recvd_mcmember_rec) ||
+		    !mtu_is_valid(p_sa_mad, p_recvd_mcmember_rec)) {
 			osm_sa_send_error(sa, p_madw,
 					  IB_SA_MAD_STATUS_REQ_INVALID);
 			goto Exit;
@@ -1494,6 +1584,13 @@ void osm_mcmr_rcv_process(IN void *context, IN void *data)
 		break;
 	case IB_MAD_METHOD_GET:
 	case IB_MAD_METHOD_GETTABLE:
+		if (!rate_is_valid(p_sa_mad, p_recvd_mcmember_rec) ||
+		    !mtu_is_valid(p_sa_mad, p_recvd_mcmember_rec)) {
+			osm_sa_send_error(sa, p_madw,
+					  IB_SA_MAD_STATUS_REQ_INVALID);
+			goto Exit;
+		}
+
 		/*
 		 * Querying a Multicast Group
 		 */
