@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2006-2009 Voltaire, Inc. All rights reserved.
- * Copyright (c) 2002-2007 Mellanox Technologies LTD. All rights reserved.
+ * Copyright (c) 2002-2011 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
  *
  * This software is available to you under a choice of one of two
@@ -52,6 +52,8 @@
 #include <complib/cl_passivelock.h>
 #include <complib/cl_debug.h>
 #include <complib/cl_qlist.h>
+#include <opensm/osm_file_ids.h>
+#define FILE_ID OSM_FILE_SA_MULTIPATH_RECORD_C
 #include <vendor/osm_vendor_api.h>
 #include <opensm/osm_port.h>
 #include <opensm/osm_node.h>
@@ -62,6 +64,7 @@
 #include <opensm/osm_sa.h>
 
 #define OSM_SA_MPR_MAX_NUM_PATH        127
+#define MAX_HOPS 64
 
 typedef struct osm_mpr_item {
 	cl_list_item_t list_item;
@@ -149,18 +152,18 @@ sa_multipath_rec_apply_tavor_mtu_limit(IN const ib_multipath_rec_t * p_mpr,
 static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 					      IN const ib_multipath_rec_t *
 					      p_mpr,
-					      IN const osm_port_t * p_src_port,
-					      IN const osm_port_t * p_dest_port,
+					      IN const osm_alias_guid_t * p_src_alias_guid,
+					      IN const osm_alias_guid_t * p_dest_alias_guid,
 					      IN const uint16_t dest_lid_ho,
 					      IN const ib_net64_t comp_mask,
 					      OUT osm_path_parms_t * p_parms)
 {
 	const osm_node_t *p_node;
-	const osm_physp_t *p_physp;
+	const osm_physp_t *p_physp, *p_physp0;
 	const osm_physp_t *p_src_physp;
 	const osm_physp_t *p_dest_physp;
 	const osm_prtn_t *p_prtn = NULL;
-	const ib_port_info_t *p_pi;
+	const ib_port_info_t *p_pi, *p_pi0;
 	ib_slvl_table_t *p_slvl_tbl;
 	ib_api_status_t status = IB_SUCCESS;
 	uint8_t mtu;
@@ -182,13 +185,14 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 
 	dest_lid = cl_hton16(dest_lid_ho);
 
-	p_dest_physp = p_dest_port->p_physp;
-	p_physp = p_src_port->p_physp;
+	p_dest_physp = p_dest_alias_guid->p_base_port->p_physp;
+	p_physp = p_src_alias_guid->p_base_port->p_physp;
 	p_src_physp = p_physp;
 	p_pi = &p_physp->port_info;
 
 	mtu = ib_port_info_get_mtu_cap(p_pi);
-	rate = ib_port_info_compute_rate(p_pi);
+	rate = ib_port_info_compute_rate(p_pi,
+					 p_pi->capability_mask & IB_PORT_CAP_HAS_EXT_SPEEDS);
 
 	/*
 	   Mellanox Tavor device performance is better using 1K MTU.
@@ -197,8 +201,10 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 	   port MTU with 1K.
 	 */
 	if (sa->p_subn->opt.enable_quirks &&
-	    sa_multipath_rec_apply_tavor_mtu_limit(p_mpr, p_src_port,
-						   p_dest_port, comp_mask))
+	    sa_multipath_rec_apply_tavor_mtu_limit(p_mpr,
+						   p_src_alias_guid->p_base_port,
+						   p_dest_alias_guid->p_base_port,
+						   comp_mask))
 		if (mtu > IB_MTU_LEN_1024) {
 			mtu = IB_MTU_LEN_1024;
 			OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
@@ -224,8 +230,9 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 		p_physp = osm_switch_get_route_by_lid(p_node->sw, dest_lid);
 		if (p_physp == 0) {
 			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4514: "
-				"Can't find routing to LID %u from switch for GUID 0x%016"
-				PRIx64 "\n", dest_lid_ho,
+				"Can't find routing to LID %u on switch %s "
+				"(GUID 0x%016"PRIx64")\n", dest_lid_ho,
+				p_node->print_desc,
 				cl_ntoh64(osm_node_get_node_guid(p_node)));
 			status = IB_NOT_FOUND;
 			goto Exit;
@@ -268,8 +275,9 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 
 		if (p_dest_physp == 0) {
 			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4515: "
-				"Can't find routing to LID %u from switch for GUID 0x%016"
-				PRIx64 "\n", dest_lid_ho,
+				"Can't find routing to LID %u on switch %s "
+				"(GUID 0x%016"PRIx64")\n", dest_lid_ho,
+				p_node->print_desc,
 				cl_ntoh64(osm_node_get_node_guid(p_node)));
 			status = IB_NOT_FOUND;
 			goto Exit;
@@ -283,19 +291,40 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 
 	while (p_physp != p_dest_physp) {
 
+		int tmp_pnum = p_physp->port_num;
 		p_node = osm_physp_get_node_ptr(p_physp);
 		p_physp = osm_physp_get_remote(p_physp);
 
 		if (p_physp == 0) {
 			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4505: "
-				"Can't find remote phys port when routing to LID %u from node GUID 0x%016"
-				PRIx64 "\n", dest_lid_ho,
-				cl_ntoh64(osm_node_get_node_guid(p_node)));
+				"Can't find remote phys port of %s (GUID "
+				"0x%016"PRIx64") port %d "
+				"while routing to LID %u",
+				p_node->print_desc,
+				cl_ntoh64(osm_node_get_node_guid(p_node)),
+				tmp_pnum,
+				dest_lid_ho);
 			status = IB_ERROR;
 			goto Exit;
 		}
 
+		/* update number of hops traversed */
 		hops++;
+		if (hops > MAX_HOPS) {
+			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4520: "
+				"Path from GUID 0x%016" PRIx64 " (%s) to"
+				" lid %u GUID 0x%016" PRIx64 " (%s) needs"
+				" more than %d hops, max %d hops allowed\n",
+				cl_ntoh64(osm_physp_get_port_guid(p_src_physp)),
+				p_src_physp->p_node->print_desc, dest_lid_ho,
+				cl_ntoh64(osm_physp_get_port_guid
+					  (p_dest_physp)),
+				p_dest_physp->p_node->print_desc, hops,
+				MAX_HOPS);
+			status = IB_NOT_FOUND;
+			goto Exit;
+		}
+
 		in_port_num = osm_physp_get_port_num(p_physp);
 
 		/*
@@ -313,7 +342,18 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 			   the destination by now!
 			 */
 			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4503: "
-				"Internal error, bad path\n");
+				"Internal error, bad path while routing "
+				"from %s (GUID: 0x%016"PRIx64") port %d "
+				"to %s (GUID: 0x%016"PRIx64") port %d; "
+				"ended at %s port %d\n",
+				p_src_alias_guid->p_base_port->p_node->print_desc,
+				cl_ntoh64(p_src_alias_guid->p_base_port->p_node->node_info.node_guid),
+				p_src_alias_guid->p_base_port->p_physp->port_num,
+				p_dest_alias_guid->p_base_port->p_node->print_desc,
+				cl_ntoh64(p_dest_alias_guid->p_base_port->p_node->node_info.node_guid),
+				p_dest_alias_guid->p_base_port->p_physp->port_num,
+				p_node->print_desc,
+				p_physp->port_num);
 			status = IB_ERROR;
 			goto Exit;
 		}
@@ -326,8 +366,13 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 		if (mtu > ib_port_info_get_mtu_cap(p_pi))
 			mtu = ib_port_info_get_mtu_cap(p_pi);
 
-		if (rate > ib_port_info_compute_rate(p_pi))
-			rate = ib_port_info_compute_rate(p_pi);
+		p_physp0 = osm_node_get_physp_ptr((osm_node_t *)p_node, 0);
+		p_pi0 = &p_physp0->port_info;
+		if (ib_path_compare_rates(rate,
+					  ib_port_info_compute_rate(p_pi,
+								    p_pi0->capability_mask & IB_PORT_CAP_HAS_EXT_SPEEDS)) > 0)
+			rate = ib_port_info_compute_rate(p_pi,
+							 p_pi0->capability_mask & IB_PORT_CAP_HAS_EXT_SPEEDS);
 
 		/*
 		   Continue with the egress port on this switch.
@@ -335,9 +380,11 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 		p_physp = osm_switch_get_route_by_lid(p_node->sw, dest_lid);
 		if (p_physp == 0) {
 			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4516: "
-				"Dead end on path to LID %u from switch for GUID 0x%016"
-				PRIx64 "\n", dest_lid_ho,
-				cl_ntoh64(osm_node_get_node_guid(p_node)));
+				"Dead end path on switch "
+				"%s (GUID: 0x%016"PRIx64") to LID %u\n",
+				p_node->print_desc,
+				cl_ntoh64(osm_node_get_node_guid(p_node)),
+				dest_lid_ho);
 			status = IB_ERROR;
 			goto Exit;
 		}
@@ -347,8 +394,13 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 		if (mtu > ib_port_info_get_mtu_cap(p_pi))
 			mtu = ib_port_info_get_mtu_cap(p_pi);
 
-		if (rate > ib_port_info_compute_rate(p_pi))
-			rate = ib_port_info_compute_rate(p_pi);
+		p_physp0 = osm_node_get_physp_ptr((osm_node_t *)p_node, 0);
+		p_pi0 = &p_physp0->port_info;
+		if (ib_path_compare_rates(rate,
+					  ib_port_info_compute_rate(p_pi,
+								    p_pi0->capability_mask & IB_PORT_CAP_HAS_EXT_SPEEDS)) > 0)
+			rate = ib_port_info_compute_rate(p_pi,
+							 p_pi0->capability_mask & IB_PORT_CAP_HAS_EXT_SPEEDS);
 
 		if (sa->p_subn->opt.qos) {
 			/*
@@ -380,8 +432,11 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 	if (mtu > ib_port_info_get_mtu_cap(p_pi))
 		mtu = ib_port_info_get_mtu_cap(p_pi);
 
-	if (rate > ib_port_info_compute_rate(p_pi))
-		rate = ib_port_info_compute_rate(p_pi);
+	if (ib_path_compare_rates(rate,
+				  ib_port_info_compute_rate(p_pi,
+							    p_pi->capability_mask & IB_PORT_CAP_HAS_EXT_SPEEDS)) > 0)
+		rate = ib_port_info_compute_rate(p_pi,
+						 p_pi->capability_mask & IB_PORT_CAP_HAS_EXT_SPEEDS);
 
 	OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
 		"Path min MTU = %u, min rate = %u\n", mtu, rate);
@@ -406,7 +461,7 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 			mtu = p_qos_level->mtu_limit;
 
 		if (p_qos_level->rate_limit_set
-		    && (rate > p_qos_level->rate_limit))
+		    && (ib_path_compare_rates(rate, p_qos_level->rate_limit) > 0))
 			rate = p_qos_level->rate_limit;
 
 		if (p_qos_level->sl_set) {
@@ -470,23 +525,22 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 		required_rate = ib_multipath_rec_rate(p_mpr);
 		switch (ib_multipath_rec_rate_sel(p_mpr)) {
 		case 0:	/* must be greater than */
-			if (rate <= required_rate)
+			if (ib_path_compare_rates(rate, required_rate) <= 0)
 				status = IB_NOT_FOUND;
 			break;
 
 		case 1:	/* must be less than */
-			if (rate >= required_rate) {
+			if (ib_path_compare_rates(rate, required_rate) >= 0) {
 				/* adjust the rate to use the highest rate
 				   lower then the required one */
-				if (required_rate > 2)
-					rate = required_rate - 1;
-				else
+				rate = ib_path_rate_get_prev(required_rate);
+				if (!rate)
 					status = IB_NOT_FOUND;
 			}
 			break;
 
 		case 2:	/* exact match */
-			if (rate < required_rate)
+			if (ib_path_compare_rates(rate, required_rate))
 				status = IB_NOT_FOUND;
 			else
 				rate = required_rate;
@@ -509,7 +563,7 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 	/* Verify the pkt_life_time */
 	/* According to spec definition IBA 1.2 Table 205 PacketLifeTime description,
 	   for loopback paths, packetLifeTime shall be zero. */
-	if (p_src_port == p_dest_port)
+	if (p_src_alias_guid->p_base_port == p_dest_alias_guid->p_base_port)
 		pkt_life = 0;	/* loopback */
 	else if (p_qos_level && p_qos_level->pkt_life_set)
 		pkt_life = p_qos_level->pkt_life;
@@ -566,7 +620,8 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 	if (comp_mask & IB_MPR_COMPMASK_RAWTRAFFIC &&
 	    cl_ntoh32(p_mpr->hop_flow_raw) & (1 << 31))
 		required_pkey =
-		    osm_physp_find_common_pkey(p_src_physp, p_dest_physp);
+		    osm_physp_find_common_pkey(p_src_physp, p_dest_physp,
+					       sa->p_subn->opt.allow_both_pkeys);
 
 	else if (comp_mask & IB_MPR_COMPMASK_PKEY) {
 		/*
@@ -578,21 +633,39 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 		 */
 		required_pkey = p_mpr->pkey;
 		if (!osm_physp_share_this_pkey
-		    (p_src_physp, p_dest_physp, required_pkey)) {
+		    (p_src_physp, p_dest_physp, required_pkey,
+		     sa->p_subn->opt.allow_both_pkeys)) {
 			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4518: "
-				"Ports do not share specified PKey 0x%04x\n"
-				"\t\tsrc %" PRIx64 " dst %" PRIx64 "\n",
-				cl_ntoh16(required_pkey),
+				"Ports src 0x%016"PRIx64" (%s port %d) "
+				"and dst 0x%016"PRIx64" (%s port %d) "
+				"do not share the specified PKey 0x%04x\n",
 				cl_ntoh64(osm_physp_get_port_guid(p_src_physp)),
+				p_src_physp->p_node->print_desc,
+				p_src_physp->port_num,
 				cl_ntoh64(osm_physp_get_port_guid
-					  (p_dest_physp)));
+					  (p_dest_physp)),
+				p_dest_physp->p_node->print_desc,
+				p_dest_physp->port_num,
+				cl_ntoh16(required_pkey));
 			status = IB_NOT_FOUND;
 			goto Exit;
 		}
 		if (p_qos_level && p_qos_level->pkey_range_len &&
 		    !osm_qos_level_has_pkey(p_qos_level, required_pkey)) {
 			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 451C: "
-				"Ports do not share PKeys defined by QoS level\n");
+				"Ports src 0x%016"PRIx64" (%s port %d) "
+				"and dst 0x%016"PRIx64" (%s port %d) "
+				"do not share specified PKey (0x%04x) as "
+				"defined by QoS level \"%s\"\n",
+				cl_ntoh64(osm_physp_get_port_guid(p_src_physp)),
+				p_src_physp->p_node->print_desc,
+				p_src_physp->port_num,
+				cl_ntoh64(osm_physp_get_port_guid
+					  (p_dest_physp)),
+				p_dest_physp->p_node->print_desc,
+				p_dest_physp->port_num,
+				cl_ntoh16(required_pkey),
+				p_qos_level->name);
 			status = IB_NOT_FOUND;
 			goto Exit;
 		}
@@ -604,10 +677,22 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 		 */
 		required_pkey = osm_qos_level_get_shared_pkey(p_qos_level,
 							      p_src_physp,
-							      p_dest_physp);
+							      p_dest_physp,
+							      sa->p_subn->opt.allow_both_pkeys);
 		if (!required_pkey) {
 			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 451D: "
-				"Ports do not share PKeys defined by QoS level\n");
+				"Ports src 0x%016"PRIx64" (%s port %d) "
+				"and dst 0x%016"PRIx64" (%s port %d) "
+				"do not share a PKey as defined by QoS "
+				"level \"%s\"\n",
+				cl_ntoh64(osm_physp_get_port_guid(p_src_physp)),
+				p_src_physp->p_node->print_desc,
+				p_src_physp->port_num,
+				cl_ntoh64(osm_physp_get_port_guid
+					  (p_dest_physp)),
+				p_dest_physp->p_node->print_desc,
+				p_dest_physp->port_num,
+				p_qos_level->name);
 			status = IB_NOT_FOUND;
 			goto Exit;
 		}
@@ -618,14 +703,20 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 		 * Just get any shared pkey.
 		 */
 		required_pkey =
-		    osm_physp_find_common_pkey(p_src_physp, p_dest_physp);
+		    osm_physp_find_common_pkey(p_src_physp, p_dest_physp,
+					       sa->p_subn->opt.allow_both_pkeys);
 		if (!required_pkey) {
 			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4519: "
-				"Ports do not have any shared PKeys\n"
-				"\t\tsrc %" PRIx64 " dst %" PRIx64 "\n",
-				cl_ntoh64(osm_physp_get_port_guid(p_physp)),
+				"Ports src 0x%016"PRIx64" (%s port %d) "
+				"and dst 0x%016"PRIx64" (%s port %d) "
+				"do not have any shared PKeys\n",
+				cl_ntoh64(osm_physp_get_port_guid(p_src_physp)),
+				p_src_physp->p_node->print_desc,
+				p_src_physp->port_num,
 				cl_ntoh64(osm_physp_get_port_guid
-					  (p_dest_physp)));
+					  (p_dest_physp)),
+				p_dest_physp->p_node->print_desc,
+				p_dest_physp->port_num);
 			status = IB_NOT_FOUND;
 			goto Exit;
 		}
@@ -654,9 +745,15 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 		if (p_qos_level && p_qos_level->sl_set &&
 		    p_qos_level->sl != required_sl) {
 			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 451E: "
-				"QoS constraints: required MultiPathRecord SL (%u) "
-				"doesn't match QoS policy SL (%u)\n",
-				required_sl, p_qos_level->sl);
+				"QoS constraints: required MultiPathRecord SL "
+				"(%u) doesn't match QoS policy \"%s\" SL (%u) "
+				"[%s port %d <-> %s port %d]\n", required_sl,
+				p_qos_level->name,
+				p_qos_level->sl,
+				p_src_alias_guid->p_base_port->p_node->print_desc,
+				p_src_alias_guid->p_base_port->p_physp->port_num,
+				p_dest_alias_guid->p_base_port->p_node->print_desc,
+				p_dest_alias_guid->p_base_port->p_physp->port_num);
 			status = IB_NOT_FOUND;
 			goto Exit;
 		}
@@ -686,8 +783,14 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 			/* this may be possible when pkey tables are created somehow in
 			   previous runs or things are going wrong here */
 			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 451A: "
-				"No partition found for PKey 0x%04x - using default SL %d\n",
-				cl_ntoh16(required_pkey), required_sl);
+				"No partition found for PKey 0x%04x - "
+				"using default SL %d "
+				"[%s port %d <-> %s port %d]\n",
+				cl_ntoh16(required_pkey), required_sl,
+				p_src_alias_guid->p_base_port->p_node->print_desc,
+				p_src_alias_guid->p_base_port->p_physp->port_num,
+				p_dest_alias_guid->p_base_port->p_node->print_desc,
+				p_dest_alias_guid->p_base_port->p_physp->port_num);
 		} else
 			required_sl = p_prtn->sl;
 
@@ -705,7 +808,13 @@ static ib_api_status_t mpr_rcv_get_path_parms(IN osm_sa_t * sa,
 
 	if (sa->p_subn->opt.qos && !(valid_sl_mask & (1 << required_sl))) {
 		OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 451F: "
-			"Selected SL (%u) leads to VL15\n", required_sl);
+			"Selected SL (%u) leads to VL15 "
+			"[%s port %d <-> %s port %d]\n",
+			required_sl,
+			p_src_alias_guid->p_base_port->p_node->print_desc,
+			p_src_alias_guid->p_base_port->p_physp->port_num,
+			p_dest_alias_guid->p_base_port->p_node->print_desc,
+			p_dest_alias_guid->p_base_port->p_physp->port_num);
 		status = IB_NOT_FOUND;
 		goto Exit;
 	}
@@ -732,20 +841,20 @@ Exit:
 	return status;
 }
 
-static void mpr_rcv_build_pr(IN osm_sa_t * sa, IN const osm_port_t * p_src_port,
-			     IN const osm_port_t * p_dest_port,
+static void mpr_rcv_build_pr(IN osm_sa_t * sa,
+			     IN const osm_alias_guid_t * p_src_alias_guid,
+			     IN const osm_alias_guid_t * p_dest_alias_guid,
 			     IN uint16_t src_lid_ho, IN uint16_t dest_lid_ho,
 			     IN uint8_t preference,
 			     IN const osm_path_parms_t * p_parms,
 			     OUT ib_path_rec_t * p_pr)
 {
-	const osm_physp_t *p_src_physp;
-	const osm_physp_t *p_dest_physp;
+	const osm_physp_t *p_src_physp, *p_dest_physp;
 
 	OSM_LOG_ENTER(sa->p_log);
 
-	p_src_physp = p_src_port->p_physp;
-	p_dest_physp = p_dest_port->p_physp;
+	p_src_physp = p_src_alias_guid->p_base_port->p_physp;
+	p_dest_physp = p_dest_alias_guid->p_base_port->p_physp;
 
 	p_pr->dgid.unicast.prefix = osm_physp_get_subnet_prefix(p_dest_physp);
 	p_pr->dgid.unicast.interface_id = osm_physp_get_port_guid(p_dest_physp);
@@ -766,7 +875,7 @@ static void mpr_rcv_build_pr(IN osm_sa_t * sa, IN const osm_port_t * p_src_port,
 
 	/* According to 1.2 spec definition Table 205 PacketLifeTime description,
 	   for loopback paths, packetLifeTime shall be zero. */
-	if (p_src_port == p_dest_port)
+	if (p_src_alias_guid->p_base_port == p_dest_alias_guid->p_base_port)
 		p_pr->pkt_life = 0x80;	/* loopback */
 	else
 		p_pr->pkt_life = (uint8_t) (p_parms->pkt_life | 0x80);
@@ -783,10 +892,10 @@ static void mpr_rcv_build_pr(IN osm_sa_t * sa, IN const osm_port_t * p_src_port,
 static osm_mpr_item_t *mpr_rcv_get_lid_pair_path(IN osm_sa_t * sa,
 						 IN const ib_multipath_rec_t *
 						 p_mpr,
-						 IN const osm_port_t *
-						 p_src_port,
-						 IN const osm_port_t *
-						 p_dest_port,
+						 IN const osm_alias_guid_t *
+						 p_src_alias_guid,
+						 IN const osm_alias_guid_t *
+						 p_dest_alias_guid,
 						 IN const uint16_t src_lid_ho,
 						 IN const uint16_t dest_lid_ho,
 						 IN const ib_net64_t comp_mask,
@@ -810,8 +919,9 @@ static osm_mpr_item_t *mpr_rcv_get_lid_pair_path(IN osm_sa_t * sa,
 	}
 	memset(p_pr_item, 0, sizeof(*p_pr_item));
 
-	status = mpr_rcv_get_path_parms(sa, p_mpr, p_src_port, p_dest_port,
-					dest_lid_ho, comp_mask, &path_parms);
+	status = mpr_rcv_get_path_parms(sa, p_mpr, p_src_alias_guid,
+					p_dest_alias_guid, dest_lid_ho,
+					comp_mask, &path_parms);
 
 	if (status != IB_SUCCESS) {
 		free(p_pr_item);
@@ -820,8 +930,8 @@ static osm_mpr_item_t *mpr_rcv_get_lid_pair_path(IN osm_sa_t * sa,
 	}
 
 	/* now try the reversible path */
-	rev_path_status = mpr_rcv_get_path_parms(sa, p_mpr, p_dest_port,
-						 p_src_port, src_lid_ho,
+	rev_path_status = mpr_rcv_get_path_parms(sa, p_mpr, p_dest_alias_guid,
+						 p_src_alias_guid, src_lid_ho,
 						 comp_mask, &rev_path_parms);
 	path_parms.reversible = (rev_path_status == IB_SUCCESS);
 
@@ -842,12 +952,13 @@ static osm_mpr_item_t *mpr_rcv_get_lid_pair_path(IN osm_sa_t * sa,
 		}
 	}
 
-	p_pr_item->p_src_port = p_src_port;
-	p_pr_item->p_dest_port = p_dest_port;
+	p_pr_item->p_src_port = p_src_alias_guid->p_base_port;
+	p_pr_item->p_dest_port = p_dest_alias_guid->p_base_port;
 	p_pr_item->hops = path_parms.hops;
 
-	mpr_rcv_build_pr(sa, p_src_port, p_dest_port, src_lid_ho, dest_lid_ho,
-			 preference, &path_parms, &p_pr_item->path_rec);
+	mpr_rcv_build_pr(sa, p_src_alias_guid, p_dest_alias_guid, src_lid_ho,
+			 dest_lid_ho, preference, &path_parms,
+			 &p_pr_item->path_rec);
 
 Exit:
 	OSM_LOG_EXIT(sa->p_log);
@@ -857,8 +968,8 @@ Exit:
 static uint32_t mpr_rcv_get_port_pair_paths(IN osm_sa_t * sa,
 					    IN const ib_multipath_rec_t * p_mpr,
 					    IN const osm_port_t * p_req_port,
-					    IN const osm_port_t * p_src_port,
-					    IN const osm_port_t * p_dest_port,
+					    IN const osm_alias_guid_t * p_src_alias_guid,
+					    IN const osm_alias_guid_t * p_dest_alias_guid,
 					    IN const uint32_t rem_paths,
 					    IN const ib_net64_t comp_mask,
 					    IN cl_qlist_t * p_list)
@@ -878,15 +989,20 @@ static uint32_t mpr_rcv_get_port_pair_paths(IN osm_sa_t * sa,
 
 	OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
 		"Src port 0x%016" PRIx64 ", Dst port 0x%016" PRIx64 "\n",
-		cl_ntoh64(osm_port_get_guid(p_src_port)),
-		cl_ntoh64(osm_port_get_guid(p_dest_port)));
+		cl_ntoh64(p_src_alias_guid->alias_guid),
+		cl_ntoh64(p_dest_alias_guid->alias_guid));
 
 	/* Check that the req_port, src_port and dest_port all share a
 	   pkey. The check is done on the default physical port of the ports. */
-	if (osm_port_share_pkey(sa->p_log, p_req_port, p_src_port) == FALSE
+	if (osm_port_share_pkey(sa->p_log, p_req_port,
+				p_src_alias_guid->p_base_port,
+				sa->p_subn->opt.allow_both_pkeys) == FALSE
 	    || osm_port_share_pkey(sa->p_log, p_req_port,
-				   p_dest_port) == FALSE
-	    || osm_port_share_pkey(sa->p_log, p_src_port, p_dest_port) == FALSE)
+				   p_dest_alias_guid->p_base_port,
+				   sa->p_subn->opt.allow_both_pkeys) == FALSE
+	    || osm_port_share_pkey(sa->p_log, p_src_alias_guid->p_base_port,
+				   p_dest_alias_guid->p_base_port,
+				   sa->p_subn->opt.allow_both_pkeys) == FALSE)
 		/* One of the pairs doesn't share a pkey so the path is disqualified. */
 		goto Exit;
 
@@ -935,9 +1051,10 @@ static uint32_t mpr_rcv_get_port_pair_paths(IN osm_sa_t * sa,
 	   redundancy, but I'm not going to bother with that now.
 	 */
 
-	osm_port_get_lid_range_ho(p_src_port, &src_lid_min_ho, &src_lid_max_ho);
-	osm_port_get_lid_range_ho(p_dest_port, &dest_lid_min_ho,
-				  &dest_lid_max_ho);
+	osm_port_get_lid_range_ho(p_src_alias_guid->p_base_port,
+				  &src_lid_min_ho, &src_lid_max_ho);
+	osm_port_get_lid_range_ho(p_dest_alias_guid->p_base_port,
+				  &dest_lid_min_ho, &dest_lid_max_ho);
 
 	OSM_LOG(sa->p_log, OSM_LOG_DEBUG, "Src LID [%u-%u], Dest LID [%u-%u]\n",
 		src_lid_min_ho, src_lid_max_ho,
@@ -955,10 +1072,11 @@ static uint32_t mpr_rcv_get_port_pair_paths(IN osm_sa_t * sa,
 		/*
 		   These paths are "fully redundant"
 		 */
-		p_pr_item = mpr_rcv_get_lid_pair_path(sa, p_mpr, p_src_port,
-						      p_dest_port, src_lid_ho,
-						      dest_lid_ho, comp_mask,
-						      preference);
+		p_pr_item = mpr_rcv_get_lid_pair_path(sa, p_mpr,
+						      p_src_alias_guid,
+						      p_dest_alias_guid,
+						      src_lid_ho, dest_lid_ho,
+						      comp_mask, preference);
 
 		if (p_pr_item) {
 			cl_qlist_insert_tail(p_list, &p_pr_item->list_item);
@@ -1018,10 +1136,11 @@ static uint32_t mpr_rcv_get_port_pair_paths(IN osm_sa_t * sa,
 		if (src_offset == dest_offset)
 			continue;	/* already reported */
 
-		p_pr_item = mpr_rcv_get_lid_pair_path(sa, p_mpr, p_src_port,
-						      p_dest_port, src_lid_ho,
-						      dest_lid_ho, comp_mask,
-						      preference);
+		p_pr_item = mpr_rcv_get_lid_pair_path(sa, p_mpr,
+						      p_src_alias_guid,
+						      p_dest_alias_guid,
+						      src_lid_ho, dest_lid_ho,
+						      comp_mask, preference);
 
 		if (p_pr_item) {
 			cl_qlist_insert_tail(p_list, &p_pr_item->list_item);
@@ -1041,10 +1160,10 @@ static osm_mpr_item_t *mpr_rcv_get_apm_port_pair_paths(IN osm_sa_t * sa,
 						       IN const
 						       ib_multipath_rec_t *
 						       p_mpr,
-						       IN const osm_port_t *
-						       p_src_port,
-						       IN const osm_port_t *
-						       p_dest_port,
+						       IN const osm_alias_guid_t *
+						       p_src_alias_guid,
+						       IN const osm_alias_guid_t *
+						       p_dest_alias_guid,
 						       IN int base_offs,
 						       IN const ib_net64_t
 						       comp_mask,
@@ -1064,12 +1183,14 @@ static osm_mpr_item_t *mpr_rcv_get_apm_port_pair_paths(IN osm_sa_t * sa,
 
 	OSM_LOG(sa->p_log, OSM_LOG_DEBUG, "Src port 0x%016" PRIx64 ", "
 		"Dst port 0x%016" PRIx64 ", base offs %d\n",
-		cl_ntoh64(osm_port_get_guid(p_src_port)),
-		cl_ntoh64(osm_port_get_guid(p_dest_port)), base_offs);
+		cl_ntoh64(p_src_alias_guid->alias_guid),
+		cl_ntoh64(p_dest_alias_guid->alias_guid),
+		base_offs);
 
-	osm_port_get_lid_range_ho(p_src_port, &src_lid_min_ho, &src_lid_max_ho);
-	osm_port_get_lid_range_ho(p_dest_port, &dest_lid_min_ho,
-				  &dest_lid_max_ho);
+	osm_port_get_lid_range_ho(p_src_alias_guid->p_base_port,
+				  &src_lid_min_ho, &src_lid_max_ho);
+	osm_port_get_lid_range_ho(p_dest_alias_guid->p_base_port,
+				  &dest_lid_min_ho, &dest_lid_max_ho);
 
 	src_lid_ho = src_lid_min_ho;
 	dest_lid_ho = dest_lid_min_ho;
@@ -1092,10 +1213,11 @@ static osm_mpr_item_t *mpr_rcv_get_apm_port_pair_paths(IN osm_sa_t * sa,
 		/*
 		   These paths are "fully redundant"
 		 */
-		p_pr_item = mpr_rcv_get_lid_pair_path(sa, p_mpr, p_src_port,
-						      p_dest_port, src_lid_ho,
-						      dest_lid_ho, comp_mask,
-						      0);
+		p_pr_item = mpr_rcv_get_lid_pair_path(sa, p_mpr,
+						      p_src_alias_guid,
+						      p_dest_alias_guid,
+						      src_lid_ho, dest_lid_ho,
+						      comp_mask, 0);
 
 		if (p_pr_item) {
 			OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
@@ -1117,9 +1239,9 @@ static osm_mpr_item_t *mpr_rcv_get_apm_port_pair_paths(IN osm_sa_t * sa,
 
 static ib_net16_t mpr_rcv_get_gids(IN osm_sa_t * sa, IN const ib_gid_t * gids,
 				   IN int ngids, IN int is_sgid,
-				   OUT osm_port_t ** pp_port)
+				   OUT osm_alias_guid_t ** pp_alias_guid)
 {
-	osm_port_t *p_port;
+	osm_alias_guid_t *p_alias_guid;
 	ib_net16_t ib_status = IB_SUCCESS;
 	int i;
 
@@ -1146,10 +1268,10 @@ static ib_net16_t mpr_rcv_get_gids(IN osm_sa_t * sa, IN const ib_gid_t * gids,
 			}
 		}
 
-		p_port =
-		    osm_get_port_by_guid(sa->p_subn,
-					 gids->unicast.interface_id);
-		if (!p_port) {
+		p_alias_guid =
+		    osm_get_alias_guid_by_guid(sa->p_subn,
+					       gids->unicast.interface_id);
+		if (!p_alias_guid) {
 			/*
 			   This 'error' is the client's fault (bad gid) so
 			   don't enter it as an error in our own log.
@@ -1163,7 +1285,7 @@ static ib_net16_t mpr_rcv_get_gids(IN osm_sa_t * sa, IN const ib_gid_t * gids,
 			goto Exit;
 		}
 
-		pp_port[i] = p_port;
+		pp_alias_guid[i] = p_alias_guid;
 	}
 
 Exit:
@@ -1174,7 +1296,7 @@ Exit:
 
 static ib_net16_t mpr_rcv_get_end_points(IN osm_sa_t * sa,
 					 IN const osm_madw_t * p_madw,
-					 OUT osm_port_t ** pp_ports,
+					 OUT osm_alias_guid_t ** pp_alias_guids,
 					 OUT int *nsrc, OUT int *ndest)
 {
 	const ib_multipath_rec_t *p_mpr;
@@ -1205,7 +1327,7 @@ static ib_net16_t mpr_rcv_get_end_points(IN osm_sa_t * sa,
 		*nsrc = p_mpr->sgid_count;
 		if (*nsrc > IB_MULTIPATH_MAX_GIDS)
 			*nsrc = IB_MULTIPATH_MAX_GIDS;
-		sa_status = mpr_rcv_get_gids(sa, gids, *nsrc, 1, pp_ports);
+		sa_status = mpr_rcv_get_gids(sa, gids, *nsrc, 1, pp_alias_guids);
 		if (sa_status != IB_SUCCESS)
 			goto Exit;
 	}
@@ -1216,7 +1338,7 @@ static ib_net16_t mpr_rcv_get_end_points(IN osm_sa_t * sa,
 			*ndest = IB_MULTIPATH_MAX_GIDS - *nsrc;
 		sa_status =
 		    mpr_rcv_get_gids(sa, gids + *nsrc, *ndest, 0,
-				     pp_ports + *nsrc);
+				     pp_alias_guids + *nsrc);
 	}
 
 Exit:
@@ -1230,11 +1352,11 @@ Exit:
 static void mpr_rcv_get_apm_paths(IN osm_sa_t * sa,
 				  IN const ib_multipath_rec_t * p_mpr,
 				  IN const osm_port_t * p_req_port,
-				  IN osm_port_t ** _pp_ports,
+				  IN osm_alias_guid_t ** _pp_alias_guids,
 				  IN const ib_net64_t comp_mask,
 				  IN cl_qlist_t * p_list)
 {
-	osm_port_t *pp_ports[4];
+	osm_alias_guid_t *pp_alias_guids[4];
 	osm_mpr_item_t *matrix[2][2];
 	int base_offs, src_lid_ho, dest_lid_ho;
 	int sumA, sumB, minA, minB;
@@ -1255,40 +1377,46 @@ static void mpr_rcv_get_apm_paths(IN osm_sa_t * sa,
 	 * Note that the diagonals on a Clos have the same number of hops, so it doesn't
 	 * really matter which diagonal we use.
 	 */
-	if (_pp_ports[0]->guid < _pp_ports[1]->guid) {
-		pp_ports[0] = _pp_ports[0];
-		pp_ports[1] = _pp_ports[1];
+	if (_pp_alias_guids[0]->p_base_port->guid <
+	    _pp_alias_guids[1]->p_base_port->guid) {
+		pp_alias_guids[0] = _pp_alias_guids[0];
+		pp_alias_guids[1] = _pp_alias_guids[1];
 	} else {
-		pp_ports[0] = _pp_ports[1];
-		pp_ports[1] = _pp_ports[0];
+		pp_alias_guids[0] = _pp_alias_guids[1];
+		pp_alias_guids[1] = _pp_alias_guids[0];
 	}
-	if (_pp_ports[2]->guid < _pp_ports[3]->guid) {
-		pp_ports[2] = _pp_ports[2];
-		pp_ports[3] = _pp_ports[3];
+	if (_pp_alias_guids[2]->p_base_port->guid <
+	    _pp_alias_guids[3]->p_base_port->guid) {
+		pp_alias_guids[2] = _pp_alias_guids[2];
+		pp_alias_guids[3] = _pp_alias_guids[3];
 	} else {
-		pp_ports[2] = _pp_ports[3];
-		pp_ports[3] = _pp_ports[2];
+		pp_alias_guids[2] = _pp_alias_guids[3];
+		pp_alias_guids[3] = _pp_alias_guids[2];
 	}
 
-	src_lid_ho = osm_port_get_base_lid(pp_ports[0]);
-	dest_lid_ho = osm_port_get_base_lid(pp_ports[2]);
+	src_lid_ho = osm_port_get_base_lid(pp_alias_guids[0]->p_base_port);
+	dest_lid_ho = osm_port_get_base_lid(pp_alias_guids[2]->p_base_port);
 
 	base_offs = src_lid_ho < dest_lid_ho ?
 	    hash_lids(src_lid_ho, dest_lid_ho, sa->p_subn->opt.lmc) :
 	    hash_lids(dest_lid_ho, src_lid_ho, sa->p_subn->opt.lmc);
 
 	matrix[0][0] =
-	    mpr_rcv_get_apm_port_pair_paths(sa, p_mpr, pp_ports[0], pp_ports[2],
-					    base_offs, comp_mask, p_list);
+	    mpr_rcv_get_apm_port_pair_paths(sa, p_mpr, pp_alias_guids[0],
+					    pp_alias_guids[2], base_offs,
+					    comp_mask, p_list);
 	matrix[0][1] =
-	    mpr_rcv_get_apm_port_pair_paths(sa, p_mpr, pp_ports[0], pp_ports[3],
-					    base_offs, comp_mask, p_list);
+	    mpr_rcv_get_apm_port_pair_paths(sa, p_mpr, pp_alias_guids[0],
+					    pp_alias_guids[3], base_offs,
+					    comp_mask, p_list);
 	matrix[1][0] =
-	    mpr_rcv_get_apm_port_pair_paths(sa, p_mpr, pp_ports[1], pp_ports[2],
-					    base_offs + 1, comp_mask, p_list);
+	    mpr_rcv_get_apm_port_pair_paths(sa, p_mpr, pp_alias_guids[1],
+					    pp_alias_guids[2], base_offs + 1,
+					    comp_mask, p_list);
 	matrix[1][1] =
-	    mpr_rcv_get_apm_port_pair_paths(sa, p_mpr, pp_ports[1], pp_ports[3],
-					    base_offs + 1, comp_mask, p_list);
+	    mpr_rcv_get_apm_port_pair_paths(sa, p_mpr, pp_alias_guids[1],
+					    pp_alias_guids[3], base_offs + 1,
+					    comp_mask, p_list);
 
 	OSM_LOG(sa->p_log, OSM_LOG_DEBUG, "APM matrix:\n"
 		"\t{0,0} 0x%X->0x%X (%d)\t| {0,1} 0x%X->0x%X (%d)\n"
@@ -1343,12 +1471,13 @@ static void mpr_rcv_get_apm_paths(IN osm_sa_t * sa,
 static void mpr_rcv_process_pairs(IN osm_sa_t * sa,
 				  IN const ib_multipath_rec_t * p_mpr,
 				  IN osm_port_t * p_req_port,
-				  IN osm_port_t ** pp_ports, IN const int nsrc,
-				  IN int ndest, IN ib_net64_t comp_mask,
+				  IN osm_alias_guid_t ** pp_alias_guids,
+				  IN const int nsrc, IN int ndest,
+				  IN ib_net64_t comp_mask,
 				  IN cl_qlist_t * p_list)
 {
-	osm_port_t **pp_src_port, **pp_es;
-	osm_port_t **pp_dest_port, **pp_ed;
+	osm_alias_guid_t **pp_src_alias_guid, **pp_es;
+	osm_alias_guid_t **pp_dest_alias_guid, **pp_ed;
 	uint32_t max_paths, num_paths, total_paths = 0;
 
 	OSM_LOG_ENTER(sa->p_log);
@@ -1358,14 +1487,14 @@ static void mpr_rcv_process_pairs(IN osm_sa_t * sa,
 	else
 		max_paths = OSM_SA_MPR_MAX_NUM_PATH;
 
-	for (pp_src_port = pp_ports, pp_es = pp_ports + nsrc;
-	     pp_src_port < pp_es; pp_src_port++) {
-		for (pp_dest_port = pp_es, pp_ed = pp_es + ndest;
-		     pp_dest_port < pp_ed; pp_dest_port++) {
+	for (pp_src_alias_guid = pp_alias_guids, pp_es = pp_alias_guids + nsrc;
+	     pp_src_alias_guid < pp_es; pp_src_alias_guid++) {
+		for (pp_dest_alias_guid = pp_es, pp_ed = pp_es + ndest;
+		     pp_dest_alias_guid < pp_ed; pp_dest_alias_guid++) {
 			num_paths =
 			    mpr_rcv_get_port_pair_paths(sa, p_mpr, p_req_port,
-							*pp_src_port,
-							*pp_dest_port,
+							*pp_src_alias_guid,
+							*pp_dest_alias_guid,
 							max_paths - total_paths,
 							comp_mask, p_list);
 			total_paths += num_paths;
@@ -1389,10 +1518,11 @@ void osm_mpr_rcv_process(IN void *context, IN void *data)
 	const ib_multipath_rec_t *p_mpr;
 	ib_sa_mad_t *p_sa_mad;
 	osm_port_t *requester_port;
-	osm_port_t *pp_ports[IB_MULTIPATH_MAX_GIDS];
+	osm_alias_guid_t *pp_alias_guids[IB_MULTIPATH_MAX_GIDS];
 	cl_qlist_t pr_list;
 	ib_net16_t sa_status;
 	int nsrc, ndest;
+	uint8_t rate, mtu;
 
 	OSM_LOG_ENTER(sa->p_log);
 
@@ -1419,7 +1549,7 @@ void osm_mpr_rcv_process(IN void *context, IN void *data)
 		goto Exit;
 	}
 
-	/* update the requester physical port. */
+	/* update the requester physical port */
 	requester_port = osm_get_port_by_mad_addr(sa->p_log, sa->p_subn,
 						  osm_madw_get_mad_addr_ptr
 						  (p_madw));
@@ -1429,8 +1559,33 @@ void osm_mpr_rcv_process(IN void *context, IN void *data)
 		goto Exit;
 	}
 
-	if (osm_log_is_active(sa->p_log, OSM_LOG_DEBUG))
-		osm_dump_multipath_record(sa->p_log, p_mpr, OSM_LOG_DEBUG);
+	if (OSM_LOG_IS_ACTIVE_V2(sa->p_log, OSM_LOG_DEBUG)) {
+		OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
+			"Requester port GUID 0x%" PRIx64 "\n",
+			cl_ntoh64(osm_port_get_guid(requester_port)));
+		osm_dump_multipath_record_v2(sa->p_log, p_mpr, FILE_ID, OSM_LOG_DEBUG);
+	}
+
+	/* Validatg rate if supplied */
+	if ((p_sa_mad->comp_mask & IB_MPR_COMPMASK_RATESELEC) &&
+	    (p_sa_mad->comp_mask & IB_MPR_COMPMASK_RATE)) {
+		rate = ib_multipath_rec_rate(p_mpr);
+		if (!ib_rate_is_valid(rate)) {
+			osm_sa_send_error(sa, p_madw,
+					  IB_SA_MAD_STATUS_REQ_INVALID);
+			goto Exit;
+		}
+	}
+	/* Validate MTU if supplied */
+	if ((p_sa_mad->comp_mask & IB_MPR_COMPMASK_MTUSELEC) &&
+	    (p_sa_mad->comp_mask & IB_MPR_COMPMASK_MTU)) {
+		mtu = ib_multipath_rec_mtu(p_mpr);
+		if (!ib_mtu_is_valid(mtu)) {
+			osm_sa_send_error(sa, p_madw,
+					  IB_SA_MAD_STATUS_REQ_INVALID);
+			goto Exit;
+		}
+	}
 
 	cl_qlist_init(&pr_list);
 
@@ -1440,13 +1595,14 @@ void osm_mpr_rcv_process(IN void *context, IN void *data)
 	 */
 	cl_plock_acquire(sa->p_lock);
 
-	sa_status = mpr_rcv_get_end_points(sa, p_madw, pp_ports, &nsrc, &ndest);
+	sa_status = mpr_rcv_get_end_points(sa, p_madw, pp_alias_guids,
+					   &nsrc, &ndest);
 
 	if (sa_status != IB_SA_MAD_STATUS_SUCCESS || !nsrc || !ndest) {
 		if (sa_status == IB_SA_MAD_STATUS_SUCCESS && (!nsrc || !ndest))
 			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 4512: "
-				"mpr_rcv_get_end_points failed, not enough GIDs "
-				"(nsrc %d ndest %d)\n", nsrc, ndest);
+				"mpr_rcv_get_end_points failed, # GIDs found; "
+				"src %d; dest %d)\n", nsrc, ndest);
 		cl_plock_release(sa->p_lock);
 		if (sa_status == IB_SA_MAD_STATUS_SUCCESS)
 			osm_sa_send_error(sa, p_madw,
@@ -1458,10 +1614,10 @@ void osm_mpr_rcv_process(IN void *context, IN void *data)
 
 	/* APM request */
 	if (nsrc == 2 && ndest == 2 && (p_mpr->num_path & 0x7F) == 2)
-		mpr_rcv_get_apm_paths(sa, p_mpr, requester_port, pp_ports,
+		mpr_rcv_get_apm_paths(sa, p_mpr, requester_port, pp_alias_guids,
 				      p_sa_mad->comp_mask, &pr_list);
 	else
-		mpr_rcv_process_pairs(sa, p_mpr, requester_port, pp_ports,
+		mpr_rcv_process_pairs(sa, p_mpr, requester_port, pp_alias_guids,
 				      nsrc, ndest, p_sa_mad->comp_mask,
 				      &pr_list);
 
