@@ -50,6 +50,8 @@
 #include <complib/cl_qmap.h>
 #include <complib/cl_passivelock.h>
 #include <complib/cl_debug.h>
+#include <opensm/osm_file_ids.h>
+#define FILE_ID OSM_FILE_SMINFO_RCV_C
 #include <opensm/osm_madw.h>
 #include <opensm/osm_log.h>
 #include <opensm/osm_node.h>
@@ -78,7 +80,6 @@ static void smi_rcv_process_get_request(IN osm_sm_t * sm,
 					IN const osm_madw_t * p_madw)
 {
 	uint8_t payload[IB_SMP_DATA_SIZE];
-	ib_smp_t *p_smp;
 	ib_sm_info_t *p_smi = (ib_sm_info_t *) payload;
 	ib_api_status_t status;
 	ib_sm_info_t *p_remote_smi;
@@ -90,9 +91,7 @@ static void smi_rcv_process_get_request(IN osm_sm_t * sm,
 	/* No real need to grab the lock for this function. */
 	memset(payload, 0, sizeof(payload));
 
-	p_smp = osm_madw_get_smp_ptr(p_madw);
-
-	CL_ASSERT(p_smp->method == IB_MAD_METHOD_GET);
+	CL_ASSERT(osm_madw_get_smp_ptr(p_madw)->method == IB_MAD_METHOD_GET);
 
 	p_smi->guid = sm->p_subn->sm_port_guid;
 	p_smi->act_count = cl_hton32(sm->p_subn->p_osm->stats.qp0_mads_sent);
@@ -182,7 +181,7 @@ static void smi_rcv_process_set_request(IN osm_sm_t * sm,
 		goto Exit;
 	}
 
-	CL_PLOCK_EXCL_ACQUIRE(sm->p_lock);
+	CL_PLOCK_ACQUIRE(sm->p_lock);
 
 	p_smi->guid = sm->p_subn->sm_port_guid;
 	p_smi->act_count = cl_hton32(sm->p_subn->p_osm->stats.qp0_mads_sent);
@@ -248,6 +247,8 @@ static void smi_rcv_process_set_request(IN osm_sm_t * sm,
 		goto Exit;
 	}
 
+	CL_PLOCK_RELEASE(sm->p_lock);
+
 	/* check legality of the needed transition in the SM state machine */
 	status = osm_sm_state_mgr_check_legality(sm, sm_signal);
 	if (status != IB_SUCCESS) {
@@ -261,7 +262,6 @@ static void smi_rcv_process_set_request(IN osm_sm_t * sm,
 			OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 2F08: "
 				"Error sending response (%s)\n",
 				ib_get_err_str(status));
-		CL_PLOCK_RELEASE(sm->p_lock);
 		goto Exit;
 	}
 
@@ -281,10 +281,11 @@ static void smi_rcv_process_set_request(IN osm_sm_t * sm,
 			"Received a STANDBY signal. Updating "
 			"sm_state_mgr master_guid: 0x%016" PRIx64 "\n",
 			cl_ntoh64(sm_smi->guid));
+		CL_PLOCK_EXCL_ACQUIRE(sm->p_lock);
 		sm->master_sm_guid = sm_smi->guid;
+		CL_PLOCK_RELEASE(sm->p_lock);
 	}
 
-	CL_PLOCK_RELEASE(sm->p_lock);
 	status = osm_sm_state_mgr_process(sm, sm_signal);
 
 	if (status != IB_SUCCESS)
@@ -296,9 +297,9 @@ Exit:
 	OSM_LOG_EXIT(sm->p_log);
 }
 
-static int smi_rcv_process_get_sm(IN osm_sm_t * sm,
-				  IN const osm_remote_sm_t * p_sm,
-				  boolean_t light_sweep)
+static void smi_rcv_process_get_sm(IN osm_sm_t * sm,
+				   IN const osm_remote_sm_t * p_sm,
+				   boolean_t light_sweep)
 {
 	const ib_sm_info_t *p_smi;
 
@@ -324,8 +325,8 @@ static int smi_rcv_process_get_sm(IN osm_sm_t * sm,
 			/* save on the sm the guid of the current master. */
 			OSM_LOG(sm->p_log, OSM_LOG_VERBOSE,
 				"Found master SM. Updating sm_state_mgr master_guid: 0x%016"
-				PRIx64 "\n", cl_ntoh64(p_sm->p_port->guid));
-			sm->master_sm_guid = p_sm->p_port->guid;
+				PRIx64 "\n", cl_ntoh64(p_smi->guid));
+			sm->master_sm_guid = p_smi->guid;
 			break;
 		case IB_SMINFO_STATE_DISCOVERING:
 		case IB_SMINFO_STATE_STANDBY:
@@ -337,8 +338,8 @@ static int smi_rcv_process_get_sm(IN osm_sm_t * sm,
 				OSM_LOG(sm->p_log, OSM_LOG_VERBOSE,
 					"Found higher SM. Updating sm_state_mgr master_guid:"
 					" 0x%016" PRIx64 "\n",
-					cl_ntoh64(p_sm->p_port->guid));
-				sm->master_sm_guid = p_sm->p_port->guid;
+					cl_ntoh64(p_smi->guid));
+				sm->master_sm_guid = p_smi->guid;
 			}
 			break;
 		default:
@@ -362,7 +363,7 @@ static int smi_rcv_process_get_sm(IN osm_sm_t * sm,
 		case IB_SMINFO_STATE_STANDBY:
 			/* This should be the response from the sm we are polling. */
 			/* If it is - then signal master is alive */
-			if (sm->master_sm_guid == p_sm->p_port->guid) {
+			if (sm->master_sm_guid == p_sm->smi.guid) {
 				/* Make sure that it is an SM with higher priority than us.
 				   If we started polling it when it was master, and it moved
 				   to standby - then it might be with a lower priority than
@@ -388,8 +389,22 @@ static int smi_rcv_process_get_sm(IN osm_sm_t * sm,
 				osm_sm_state_mgr_signal_master_is_alive(sm);
 			else {
 				/* This is a response we got while sweeping the subnet.
-				   We will handle a case of handover needed later on, when the sweep
-				   is done and all SMs are recongnized. */
+				 *
+				 * If this is during a heavy sweep, we will handle a case of
+				 * handover needed later on, when the sweep is done and all
+				 * SMs are recognized.
+				 *
+				 * If this is during a light sweep, initiate a heavy sweep
+				 * to initiate handover scenarios.
+				 *
+				 * Note that it does not matter if the remote SM is lower
+				 * or higher priority.  If it is lower priority, we must
+				 * wait for it HANDOVER.  If it is higher priority, we need
+				 * to HANDOVER to it.  Both cases are handled after doing
+				 * a heavy sweep.
+				 */
+				if (light_sweep)
+					sm->p_subn->force_heavy_sweep = TRUE;
 			}
 			break;
 		case IB_SMINFO_STATE_STANDBY:
@@ -408,7 +423,6 @@ static int smi_rcv_process_get_sm(IN osm_sm_t * sm,
 	}
 
 	OSM_LOG_EXIT(sm->p_log);
-	return 0;
 }
 
 static void smi_rcv_process_get_response(IN osm_sm_t * sm,
@@ -437,7 +451,7 @@ static void smi_rcv_process_get_response(IN osm_sm_t * sm,
 	p_sm_tbl = &sm->p_subn->sm_guid_tbl;
 	port_guid = p_smi->guid;
 
-	osm_dump_sm_info(sm->p_log, p_smi, OSM_LOG_DEBUG);
+	osm_dump_sm_info_v2(sm->p_log, p_smi, FILE_ID, OSM_LOG_DEBUG);
 
 	/* Check that the sm_key of the found SM is the same as ours,
 	   or is zero. If not - OpenSM cannot continue with configuration!. */
@@ -445,8 +459,8 @@ static void smi_rcv_process_get_response(IN osm_sm_t * sm,
 		OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 2F18: "
 			"Got SM with sm_key that doesn't match our "
 			"local key. Exiting\n");
-		osm_log(sm->p_log, OSM_LOG_SYS,
-			"Found remote SM with non-matching sm_key. Exiting\n");
+		osm_log_v2(sm->p_log, OSM_LOG_SYS, FILE_ID,
+			   "Found remote SM with non-matching sm_key. Exiting\n");
 		osm_exit_flag = TRUE;
 		goto Exit;
 	}
@@ -486,7 +500,7 @@ static void smi_rcv_process_get_response(IN osm_sm_t * sm,
 			goto _unlock_and_exit;
 		}
 
-		osm_remote_sm_init(p_sm, p_port, p_smi);
+		osm_remote_sm_init(p_sm, p_smi);
 
 		cl_qmap_insert(p_sm_tbl, port_guid, &p_sm->map_item);
 	} else
@@ -523,7 +537,7 @@ static void smi_rcv_process_set_response(IN osm_sm_t * sm,
 	}
 
 	p_smi = ib_smp_get_payload_ptr(p_smp);
-	osm_dump_sm_info(sm->p_log, p_smi, OSM_LOG_DEBUG);
+	osm_dump_sm_info_v2(sm->p_log, p_smi, FILE_ID, OSM_LOG_DEBUG);
 
 	/* Check the AttributeModifier */
 	if (p_smp->attr_mod != IB_SMINFO_ATTR_MOD_HANDOVER) {
@@ -551,6 +565,12 @@ void osm_sminfo_rcv_process(IN void *context, IN void *data)
 	CL_ASSERT(p_madw);
 
 	p_smp = osm_madw_get_smp_ptr(p_madw);
+	if (ib_smp_get_status(p_smp)) {
+		OSM_LOG(sm->p_log, OSM_LOG_DEBUG,
+			"MAD status 0x%x received\n",
+			cl_ntoh16(ib_smp_get_status(p_smp)));
+		goto Exit;
+	}
 
 	/* Determine if this is a request for our own SMInfo or if
 	   this is a response to our request for another SM's SMInfo. */
