@@ -48,6 +48,8 @@
 #include <ctype.h>
 #include <complib/cl_debug.h>
 #include <complib/cl_qmap.h>
+#include <opensm/osm_file_ids.h>
+#define FILE_ID OSM_FILE_UCAST_UPDN_C
 #include <opensm/osm_switch.h>
 #include <opensm/osm_opensm.h>
 #include <opensm/osm_ucast_mgr.h>
@@ -280,20 +282,18 @@ static int updn_subn_rank(IN updn_t * p_updn)
 }
 
 /* hack: preserve min hops entries to any other root switches */
-static void updn_clear_non_root_hops(updn_t * p_updn, osm_switch_t * p_sw)
+static void updn_clear_non_root_hops(updn_t * updn, osm_switch_t * sw)
 {
-	osm_port_t *p_port;
+	osm_port_t *port;
 	unsigned i;
 
-	for (i = 0; i < p_sw->num_hops; i++)
-		if (p_sw->hops[i]) {
-			p_port =
-			    cl_ptr_vector_get(&p_updn->p_osm->subn.port_lid_tbl,
-					      i);
-			if (!p_port || !p_port->p_node->sw
-			    || ((struct updn_node *)p_port->p_node->sw->priv)->
+	for (i = 0; i < sw->num_hops; i++)
+		if (sw->hops[i]) {
+			port = osm_get_port_by_lid_ho(&updn->p_osm->subn, i);
+			if (!port || !port->p_node->sw
+			    || ((struct updn_node *)port->p_node->sw->priv)->
 			    rank != 0)
-				memset(p_sw->hops[i], 0xff, p_sw->num_ports);
+				memset(sw->hops[i], 0xff, sw->num_ports);
 		}
 }
 
@@ -630,7 +630,7 @@ static int updn_lid_matrices(void *ctx)
 		ret = -1;
 	}
 
-	if (osm_log_is_active(&p_updn->p_osm->log, OSM_LOG_ROUTING))
+	if (OSM_LOG_IS_ACTIVE_V2(&p_updn->p_osm->log, OSM_LOG_ROUTING))
 		osm_dump_qmap_to_file(p_updn->p_osm, "opensm-updn-roots.dump",
 				      &p_updn->p_osm->subn.sw_guid_tbl,
 				      dump_roots, NULL);
@@ -663,7 +663,7 @@ int osm_ucast_updn_setup(struct osm_routing_engine *r, osm_opensm_t *osm)
 	updn->p_osm = osm;
 
 	r->context = updn;
-	r->delete = updn_delete;
+	r->destroy = updn_delete;
 	r->build_lid_matrices = updn_lid_matrices;
 
 	return 0;
